import strawberry
from strawberry.fastapi import GraphQLRouter

from {{cookiecutter.project_name}}.web.gql.context import get_context

{%- if cookiecutter.enable_routers == "True" %}
from {{cookiecutter.project_name}}.web.gql import echo
{%- if cookiecutter.add_dummy == 'True' %}
from {{cookiecutter.project_name}}.web.gql import dummy
{%- endif %}
{%- if cookiecutter.enable_redis == "True" %}
from {{cookiecutter.project_name}}.web.gql import redis
{%- endif %}
{%- if cookiecutter.enable_rmq == "True" %}
from {{cookiecutter.project_name}}.web.gql import rabbit
{%- endif %}
{%- if cookiecutter.enable_kafka == "True" %}
from {{cookiecutter.project_name}}.web.gql import kafka
{%- endif %}

{%- endif %}

@strawberry.type
class Query(  # noqa: WPS215
    {%- if cookiecutter.enable_routers == "True" %}
    echo.Query,
    {%- if cookiecutter.add_dummy == 'True' %}
    dummy.Query,
    {%- endif %}
    {%- if cookiecutter.enable_redis == "True" %}
    redis.Query,
    {%- endif %}
    {%- endif %}
):
    """Main query."""


@strawberry.type
class Mutation(  # noqa: WPS215
    {%- if cookiecutter.enable_routers == "True" %}
    echo.Mutation,
    {%- if cookiecutter.add_dummy == 'True' %}
    dummy.Mutation,
    {%- endif %}
    {%- if cookiecutter.enable_redis == "True" %}
    redis.Mutation,
    {%- endif %}
    {%- if cookiecutter.enable_rmq == "True" %}
    rabbit.Mutation,
    {%- endif %}
    {%- if cookiecutter.enable_kafka == "True" %}
    kafka.Mutation,
    {%- endif %}
    {%- endif %}
):
    """Main mutation."""


schema = strawberry.Schema(
    Query,
    Mutation,
)

gql_router = GraphQLRouter(
    schema,
    graphiql=True,
    context_getter=get_context,
)



import enum
from pathlib import Path
from tempfile import gettempdir
from typing import Optional

from pydantic import BaseSettings
from yarl import URL

TEMP_DIR = Path(gettempdir())

class LogLevel(str, enum.Enum):  # noqa: WPS600
    """Possible log levels."""

    NOTSET = "NOTSET"
    DEBUG = "DEBUG"
    INFO = "INFO"
    WARNING = "WARNING"
    ERROR = "ERROR"
    FATAL = "FATAL"


class Settings(BaseSettings):
    """
    Application settings.

    These parameters can be configured
    with environment variables.
    """

    host: str = "127.0.0.1"
    port: int = 8000
    # quantity of workers for uvicorn
    workers_count: int = 1
    # Enable uvicorn reloading
    reload: bool = False

    # Current environment
    environment: str = "dev"

    log_level: LogLevel = LogLevel.INFO

    {% if cookiecutter.db_info.name != "none" -%}

    # Variables for the database
    {%- if cookiecutter.db_info.name == "sqlite" %}
    db_file: Path = TEMP_DIR / "db.sqlite3"
    {%- else %}
    db_host: str = "localhost"
    db_port: int = {{cookiecutter.db_info.port}}
    db_user: str = "{{cookiecutter.project_name}}"
    db_pass: str = "{{cookiecutter.project_name}}"
    db_base: str = "{{cookiecutter.project_name}}"
    {%- endif %}
    db_echo: bool = False

    {%- endif %}


    {%- if cookiecutter.enable_redis == "True" %}

    # Variables for Redis
    redis_host: str = "{{cookiecutter.project_name}}-redis"
    redis_port: int = 6379
    redis_user: Optional[str] = None
    redis_pass: Optional[str] = None
    redis_base: Optional[int] = None

    {%- endif %}


    {%- if cookiecutter.enable_rmq == "True" %}

    # Variables for RabbitMQ
    rabbit_host: str = "{{cookiecutter.project_name}}-rmq"
    rabbit_port: int = 5672
    rabbit_user: str = "guest"
    rabbit_pass: str = "guest"
    rabbit_vhost: str = "/"

    rabbit_pool_size: int = 2
    rabbit_channel_pool_size: int = 10

    {%- endif %}


    {%- if cookiecutter.prometheus_enabled == "True" %}

    # This variable is used to define
    # multiproc_dir. It's required for [uvi|guni]corn projects.
    prometheus_dir: Path = TEMP_DIR / "prom"

    {%- endif %}


    {%- if cookiecutter.sentry_enabled == "True" %}

    # Sentry's configuration.
    sentry_dsn: Optional[str] = None
    sentry_sample_rate: float = 1.0

    {%- endif %}


    {%- if cookiecutter.otlp_enabled == "True" %}

    # Grpc endpoint for opentelemetry.
    # E.G. http://localhost:4317
    opentelemetry_endpoint: Optional[str] = None

    {%- endif %}

    {%- if cookiecutter.enable_kafka == "True" %}

    kafka_bootstrap_servers: list[str] = ["{{cookiecutter.project_name}}-kafka:9092"]

    {%- endif %}

    {%- if cookiecutter.db_info.name != "none" %}


    @property
    def db_url(self) -> URL:
        """
        Assemble database URL from settings.

        :return: database URL.
        """
        {%- if cookiecutter.db_info.name == "sqlite" %}
        return URL.build(
            {%- if cookiecutter.orm == "sqlalchemy" %}
            scheme="{{cookiecutter.db_info.async_driver}}",
            {%- elif cookiecutter.orm == "tortoise" %}
            scheme="{{cookiecutter.db_info.driver_short}}",
            {%- else %}
            scheme="{{cookiecutter.db_info.driver}}",
            {%- endif %}
            path=f"///{self.db_file}"
        )
        {%- else %}
        return URL.build(
            {%- if cookiecutter.orm == "sqlalchemy" %}
            scheme="{{cookiecutter.db_info.async_driver}}",
            {%- elif cookiecutter.orm == "tortoise" %}
            scheme="{{cookiecutter.db_info.driver_short}}",
            {%- else %}
            scheme="{{cookiecutter.db_info.driver}}",
            {%- endif %}
            host=self.db_host,
            port=self.db_port,
            user=self.db_user,
            password=self.db_pass,
            path=f"/{self.db_base}",
        )
        {%- endif %}
    {%- endif %}

    {%- if cookiecutter.enable_redis == "True" %}
    @property
    def redis_url(self) -> URL:
        """
        Assemble REDIS URL from settings.

        :return: redis URL.
        """
        path = ""
        if self.redis_base is not None:
            path = f"/{self.redis_base}"
        return URL.build(
            scheme="redis",
            host=self.redis_host,
            port=self.redis_port,
            user=self.redis_user,
            password=self.redis_pass,
            path=path,
        )
    {%- endif %}

    {%- if cookiecutter.enable_rmq == "True" %}
    @property
    def rabbit_url(self) -> URL:
        """
        Assemble RabbitMQ URL from settings.

        :return: rabbit URL.
        """
        return URL.build(
            scheme="amqp",
            host=self.rabbit_host,
            port=self.rabbit_port,
            user=self.rabbit_user,
            password=self.rabbit_pass,
            path=self.rabbit_vhost,
        )
    {%- endif %}

    class Config:
        env_file = ".env"
        env_prefix = "{{cookiecutter.project_name | upper }}_"
        env_file_encoding = "utf-8"


settings = Settings()



from aio_pika import Channel
from aio_pika.pool import Pool
from fastapi import Request


def get_rmq_channel_pool(request: Request) -> Pool[Channel]:  # pragma: no cover
    """
    Get channel pool from the state.

    :param request: current request.
    :return: channel pool.
    """
    return request.app.state.rmq_channel_pool



from fastapi_template.input_model import (
    BuilderContext,
    MenuEntry,
    SingularMenuModel,
    MultiselectMenuModel,
    BaseMenuModel,
    Database,
    SKIP_ENTRY,
)
from importlib.metadata import version
from typing import Callable, Optional
from click import Command, Option
import re

from prompt_toolkit import prompt
from prompt_toolkit.document import Document
from prompt_toolkit.validation import ValidationError, Validator
from typing import Any
from termcolor import colored


class SnakeCaseValidator(Validator):
    def validate(self, document: Document):
        text = document.text
        if not text or re.fullmatch(r"[a-zA-Z][\w\_\d]*", text) is None:
            raise ValidationError(message="Must be a valid snake_case name.")


def db_menu_update_info(ctx: BuilderContext, menu: SingularMenuModel) -> BuilderContext:
    for entry in menu.entries:
        if entry.code == ctx.db:
            ctx.db_info = entry.additional_info.dict()
    return ctx


def disable_orm(ctx: BuilderContext) -> MenuEntry:
    if ctx.db == "none":
        ctx.orm = "none"
        return SKIP_ENTRY
    return None


def do_not_ask_features_if_quite(ctx: BuilderContext) -> Optional[list[MenuEntry]]:
    if ctx.quite:
        return [SKIP_ENTRY]
    return None


def check_db(allowed_values: list[str]) -> Callable[[BuilderContext], bool]:
    def checker(ctx: BuilderContext) -> bool:
        return ctx.db not in allowed_values

    return checker


def check_orm(allowed_values: list[str]) -> Callable[[BuilderContext], bool]:
    def checker(ctx: BuilderContext) -> bool:
        return ctx.orm not in allowed_values

    return checker


api_menu = SingularMenuModel(
    title="API type",
    code="api_type",
    cli_name="api-type",
    description="Select API type for your application",
    entries=[
        MenuEntry(
            code="rest",
            user_view="REST API",
            description=(
                "Choose this option if you want to create a service with {name}.\n"
                "It's more suitable for {generic} web-services or services without databases.".format(
                    name=colored("REST API", color="green"),
                    generic=colored("generic", color="cyan", attrs=["underline"]),
                )
            ),
        ),
        MenuEntry(
            code="graphql",
            user_view="GrapQL API",
            description=(
                "Choose this option if you want to create a service with {name}.\n"
                "It's more suitable for services with {reason} and deep nesting.".format(
                    name=colored("GraphQL", color="green"),
                    reason=colored(
                        "lots of entities", color="cyan", attrs=["underline"]
                    ),
                )
            ),
        ),
    ],
)

db_menu = SingularMenuModel(
    title="Database",
    code="db",
    description="Select a database for your app",
    after_ask_fun=db_menu_update_info,
    entries=[
        MenuEntry(
            code="none",
            user_view="No database",
            description="This project doesn't need a database.",
            additional_info=Database(
                name="none",
                image=None,
                driver=None,
                async_driver=None,
                port=None,
            ),
        ),
        MenuEntry(
            code="sqlite",
            user_view="SQLite database",
            description=(
                "{name} is a good embeddable database.\n"
                "This option will be a great fit for {small} systems where you are "
                "going to run {how_many} of your application.".format(
                    name=colored("SQLite", color="green"),
                    small=colored("small", color="cyan", attrs=["underline"]),
                    how_many=colored(
                        "only one instance",
                        color="red",
                        attrs=["bold", "underline"],
                    ),
                )
            ),
            additional_info=Database(
                name="sqlite",
                image=None,
                async_driver="sqlite+aiosqlite",
                driver_short="sqlite",
                driver="sqlite",
                port=None,
            ),
        ),
        MenuEntry(
            code="mysql",
            user_view="MySQL database",
            description=(
                "{name} is the most popular database made by oracle.\n"
                "It's a good fit for {prod} application.".format(
                    name=colored("MySQL", color="green"),
                    prod=colored("production-grade", color="cyan", attrs=["underline"]),
                )
            ),
            additional_info=Database(
                name="mysql",
                image="bitnami/mysql:8.0.30",
                async_driver="mysql+aiomysql",
                driver_short="mysql",
                driver="mysql",
                port=3306,
            ),
        ),
        MenuEntry(
            code="postgresql",
            user_view="PostgreSQL database",
            description=(
                "{name} is second most popular open-source relational database.\n"
                "It's a good fit for {prod} application.".format(
                    name=colored("PostgreSQL", color="green"),
                    prod=colored("production-grade", color="cyan", attrs=["underline"]),
                )
            ),
            additional_info=Database(
                name="postgresql",
                image="postgres:13.8-bullseye",
                async_driver="postgresql+asyncpg",
                driver_short="postgres",
                driver="postgresql",
                port=5432,
            ),
        ),
    ],
)

ci_menu = SingularMenuModel(
    title="CI|CD",
    code="ci_type",
    cli_name="ci",
    description="Select a CI for your app",
    entries=[
        MenuEntry(
            code="none",
            user_view="Do not add CI/CD.",
            description="This project doesn't need to have CI/CD.",
        ),
        MenuEntry(
            code="gitlab_ci",
            user_view="Gitlab CI",
            description=(
                "Use this option if you use gitlab as your VCS.\n"
                "This option will add test jobs in your {file} file.\n"
                "({warn}).".format(
                    file=colored(
                        "`.gitlab-ci.yml`",
                        color="cyan",
                    ),
                    warn=colored(
                        "To use it please use docker or kubernetes executors",
                        color="red",
                        attrs=["underline"],
                    ),
                )
            ),
        ),
        MenuEntry(
            code="github",
            user_view="Github Actions",
            description=(
                "Use this option if you use github as your VCS.\n"
                "This option will create {file} file that adds test jobs for your project.".format(
                    file=colored(
                        "`.github/workflows/tests.yml`",
                        color="cyan",
                    )
                )
            ),
        ),
    ],
)

orm_menu = SingularMenuModel(
    title="ORM",
    code="orm",
    description="Choose Object–Relational Mapper lib",
    cli_name="orm",
    before_ask_fun=disable_orm,
    entries=[
        MenuEntry(
            code="none",
            user_view="Whithout ORMs",
            description=(
                "If you select this option, you will get only {what}.\n"
                "The rest {warn}.".format(
                    what=colored("raw database", color="green"),
                    warn=colored("is up to you", color="red", attrs=["underline"]),
                )
            ),
        ),
        MenuEntry(
            code="ormar",
            user_view="Ormar",
            description=(
                "{what} is a great {feature} ORM.\n"
                "It's compatible with pydantic models and alembic migrator.".format(
                    what=colored("Ormar", color="green"),
                    feature=colored("SQLAlchemy-based", color="cyan"),
                )
            ),
        ),
        MenuEntry(
            code="sqlalchemy",
            user_view="SQLAlchemy",
            description=(
                "{what} is the most popular python ORM.\n"
                "It has a {feature} and a big community around it.".format(
                    what=colored("SQLAlchemy", color="green"),
                    feature=colored("great documentation", color="cyan"),
                )
            ),
        ),
        MenuEntry(
            code="tortoise",
            user_view="Tortoise",
            description=(
                "{what} is a great {feature} ORM.\n"
                "It's easy to use, it has it's own migration tooling.".format(
                    what=colored("Tortoise", color="green"),
                    feature=colored("fully-async", color="cyan"),
                )
            ),
        ),
        MenuEntry(
            code="psycopg",
            user_view="PsycoPG",
            is_hidden=check_db(["postgresql"]),
            description=(
                "{what} is a {feature} for Postgresql.\n"
                "It's async and can work with pydantic types.".format(
                    what=colored("PsycoPG", color="green"),
                    feature=colored("raw driver", color="cyan"),
                )
            ),
        ),
        MenuEntry(
            code="piccolo",
            user_view="Piccolo",
            is_hidden=check_db(["postgresql", "sqlite"]),
            description=(
                "{what} is a great ORM for Postgresql and SQLite.\n"
                "It's very flexible and fully {feature}.".format(
                    what=colored("Piccolo", color="green"),
                    feature=colored("tab-completable", color="cyan"),
                )
            ),
        ),
    ],
)

features_menu = MultiselectMenuModel(
    title="Additional tweaks",
    code="features",
    description="Additional project features",
    multiselect=True,
    before_ask=do_not_ask_features_if_quite,
    entries=[
        MenuEntry(
            code="enable_redis",
            cli_name="redis",
            user_view="Add redis support",
            description=(
                "{name} is a cool and lightweight in-memory key-value store.\n"
                "It's good for {purpose1} or {purpose2}.".format(
                    name=colored(
                        "Redis",
                        color="green",
                    ),
                    purpose1=colored("caching", color="cyan"),
                    purpose2=colored("storing temporary variables", color="cyan"),
                )
            ),
        ),
        MenuEntry(
            code="enable_rmq",
            cli_name="rabbit",
            user_view="Add RabbitMQ support",
            description=(
                "{name} is a flexible message broker.\n"
                "It's used to create {purp1} systems or for {purp2}.".format(
                    name=colored("RabbitMQ", color="green"),
                    purp1=colored("event-based", color="cyan"),
                    purp2=colored("async computations", color="cyan"),
                )
            ),
        ),
        MenuEntry(
            code="enable_migrations",
            cli_name="migrations",
            user_view="Add Migrations",
            is_hidden=lambda ctx: ctx.db == "none",
            description=(
                "Add database {what} config.\n"
                "This options adds ability to {why} database schema.".format(
                    what=colored("schema migration", color="green"),
                    why=colored("automatically update", color="cyan"),
                )
            ),
        ),
        MenuEntry(
            code="enable_kube",
            cli_name="kube",
            user_view="Add kubernetes configs",
            description=(
                "This option will add {what} manifests to your project.\n"
                "But this option is {warn}, since if you want to use k8s, please create helm.".format(
                    what=colored("kubernetes", color="green"),
                    warn=colored("deprecated", color="red", attrs=["underline"]),
                )
            ),
        ),
        MenuEntry(
            code="add_dummy",
            cli_name="dummy",
            user_view="Add dummy model",
            description=(
                "This option creates {what} as an example of how to use chosen ORM.\n"
                "Also this option will generate you an example of {dao}.".format(
                    what=colored("dummy model", color="green"),
                    dao=colored("DAO class", color="cyan"),
                )
            ),
        ),
        MenuEntry(
            code="enable_routers",
            cli_name="routers",
            user_view="Add example routers",
            description=(
                "This option will add example {what} for your application.\n"
                "It checks for all enabled features and create views for them.".format(
                    what=colored("routers", color="green")
                )
            ),
        ),
        MenuEntry(
            code="self_hosted_swagger",
            cli_name="swagger",
            user_view="Add self hosted swagger",
            description=(
                "This option will place swagger UI {what} files inside the project.\n"
                "This is {crit} for systems where you cannot make external requests.".format(
                    what=colored("js and css", color="green"),
                    crit=colored("critical", color="red", attrs=["underline"]),
                )
            ),
        ),
        MenuEntry(
            code="prometheus_enabled",
            cli_name="prometheus",
            user_view="Add prometheus compatible metrics",
            description=(
                "{name} is a system that can collect metrics.\n"
                "This option will add a {path} route where you can find you app's metrics.".format(
                    name=colored("Prometheus", color="green"),
                    path=colored("`/metrics`", color="blue"),
                )
            ),
        ),
        MenuEntry(
            code="sentry_enabled",
            cli_name="sentry",
            user_view="Add sentry integration",
            description=(
                "{what} is super cool system that helps finding bugs.\n"
                "This feature will add sentry integration to your project. ({warn}).".format(
                    what=colored("Sentry", color="green"),
                    warn=colored(
                        "This option may decrease speed",
                        color="red",
                        attrs=["underline"],
                    ),
                )
            ),
        ),
        MenuEntry(
            code="enable_loguru",
            cli_name="loguru",
            user_view="Add loguru logger",
            description=(
                "{what} is a good drop-in replacement for default logger.\n"
                "It makes logs look {why}.".format(
                    what=colored("Loguru", color="green"),
                    why=colored(
                        "fancy and cool",
                        color="cyan",
                        attrs=["underline"],
                    ),
                )
            ),
        ),
        MenuEntry(
            code="otlp_enabled",
            cli_name="opentelemetry",
            user_view="Add opentelemetry integration",
            description=(
                "{what} is a new way to collect telemetry.\n"
                "It sends {why} and everything in collectors. Compatible with {comp}.".format(
                    what=colored("Opentelemetry", color="green"),
                    why=colored(
                        "traces, logs",
                        color="cyan",
                    ),
                    comp=colored(
                        "Jaeger",
                        color="cyan",
                    ),
                )
            ),
        ),
        MenuEntry(
            code="traefik_labels",
            cli_name="traefik",
            user_view="Adds traefik labels to docker container",
            description=(
                "{what} is successor of the nginx.\n"
                "This option adds {why} to docker containers and makes them {comp}.".format(
                    what=colored("Traefik", color="green"),
                    why=colored(
                        "labels",
                        color="cyan",
                    ),
                    comp=colored(
                        "discoverable",
                        color="cyan",
                    ),
                )
            ),
        ),
        MenuEntry(
            code="enable_kafka",
            cli_name="kafka",
            user_view="Add Kafka support",
            description=(
                "{what} is a message broker.\n"
                "Kafka doesn't have a fancy routing as RabbitMQ, but it's {why}.".format(
                    what=colored("Kafka", color="green"),
                    why=colored(
                        "super fast",
                        color="cyan",
                    ),
                )
            ),
        ),
    ],
)


def handle_cli(
    menus: list[BaseMenuModel],
    callback: Callable[[BuilderContext], None],
):
    def inner_callback(**cli_args: Any):
        if cli_args["version"]:
            print(version("fastapi_template"))
            exit(0)

        context = BuilderContext(**cli_args)

        if context.project_name is None:
            context.project_name = prompt(
                "Project name: ",
                validator=SnakeCaseValidator(),
            )
        context.kube_name = context.project_name.replace("_", "-")

        for menu in menus:
            if menu.need_ask(context):
                context = menu.ask(context)
                if context is None:
                    print("Project generation stopped. Goodbye!")
                    return
                context = BuilderContext(**context.dict())

            context = BuilderContext(**menu.after_ask(context=context).dict())

        callback(context)

    return inner_callback


def run_command(callback: Callable[[BuilderContext], None]) -> None:
    menus: "list[BaseMenuModel]" = [
        api_menu,
        db_menu,
        orm_menu,
        ci_menu,
        features_menu,
    ]

    cmd = Command(
        None,
        params=[
            Option(
                ["-n", "--name", "project_name"],
                help="Name of your awesome project",
            ),
            Option(
                ["-V", "--version", "version"],
                is_flag=True,
                help="Prints current version",
            ),
            Option(
                ["--force"],
                is_flag=True,
                help="Owerrite directory if it exists",
            ),
            Option(
                ["--quite"],
                is_flag=True,
                help="Do not ask for features during generation",
            ),
        ],
        callback=handle_cli(
            menus=menus,
            callback=callback,
        ),
    )
    for menu in menus:
        cmd.params.extend(menu.get_cli_options())
    cmd.main()



import strawberry
from strawberry.types import Info

from {{cookiecutter.project_name}}.web.gql.context import Context
from {{cookiecutter.project_name}}.web.gql.rabbit.schema import RabbitMessageDTO
from aio_pika import Message


@strawberry.type
class Mutation:
    """Mutation for rabbit package."""

    @strawberry.mutation(description="Send message to RabbitMQ")
    async def send_rabbit_message(
        self, message: RabbitMessageDTO, info: Info[Context, None]
    ) -> None:
        """
        Sends a message in RabbitMQ.

        :param message: message to publish.
        :param info: current context.
        """
        async with info.context.rabbit.acquire() as conn:
            exchange = await conn.declare_exchange(
                name=message.exchange_name,
                auto_delete=True,
            )
            await exchange.publish(
                message=Message(
                    body=message.message.encode("utf-8"),
                    content_encoding="utf-8",
                    content_type="text/plain",
                ),
                routing_key=message.routing_key,
            )



from tortoise import models, fields


class DummyModel(models.Model):
    """Model for demo purpose."""

    id = fields.IntField(pk=True)
    name = fields.CharField(max_length=200)  # noqa: WPS432

    def __str__(self) -> str:
        return self.name



import uuid
import pytest
from httpx import AsyncClient
from fastapi import FastAPI
from starlette import status

@pytest.mark.anyio
async def test_echo(fastapi_app: FastAPI, client: AsyncClient) -> None:
    """
    Tests that echo route works.

    :param fastapi_app: current application.
    :param client: clien for the app.
    """
    {%- if cookiecutter.api_type == 'rest' %}
    url = fastapi_app.url_path_for('send_echo_message')
    {%- elif cookiecutter.api_type == 'graphql' %}
    url = fastapi_app.url_path_for('handle_http_post')
    {%- endif %}
    message = uuid.uuid4().hex
    {%- if cookiecutter.api_type == 'rest' %}
    response = await client.post(url, json={
        "message": message
    })
    assert response.status_code == status.HTTP_200_OK
    assert response.json()['message'] == message
    {%- elif cookiecutter.api_type == 'graphql' %}
    response = await client.post(
        url,
        json={
            "query": "query($message: String!){echo(message: $message)}",
            "variables": {
                "message": message,
            },
        },
    )
    assert response.status_code == status.HTTP_200_OK
    assert response.json()['data']['echo'] == message
    {%- endif %}




import uvicorn
import os
import shutil

from {{cookiecutter.project_name}}.settings import settings


{%- if cookiecutter.prometheus_enabled == "True" %}
def set_multiproc_dir() -> None:
    """
    Sets mutiproc_dir env variable.

    This function cleans up the multiprocess directory
    and recreates it. This actions are required by prometheus-client
    to share metrics between processes.

    After cleanup, it sets two variables.
    Uppercase and lowercase because different
    versions of the prometheus-client library
    depend on different environment variables,
    so I've decided to export all needed variables,
    to avoid undefined behaviour.
    """
    shutil.rmtree(settings.prometheus_dir, ignore_errors=True)
    os.makedirs(settings.prometheus_dir, exist_ok=True)
    os.environ["prometheus_multiproc_dir"] = str(
        settings.prometheus_dir.expanduser().absolute(),
    )
    os.environ["PROMETHEUS_MULTIPROC_DIR"] = str(
        settings.prometheus_dir.expanduser().absolute(),
    )
{%- endif %}


def main() -> None:
    """Entrypoint of the application."""
    {%- if cookiecutter.prometheus_enabled == "True" %}
    set_multiproc_dir()
    {%- endif %}
    {%- if cookiecutter.orm == "piccolo" %}
    os.environ['PICCOLO_CONF'] = "{{cookiecutter.project_name}}.piccolo_conf"
    {%- endif %}
    uvicorn.run(
        "{{cookiecutter.project_name}}.web.application:get_app",
        workers=settings.workers_count,
        host=settings.host,
        port=settings.port,
        reload=settings.reload,
        log_level=settings.log_level.value.lower(),
        factory=True,
    )


if __name__ == "__main__":
    main()



from redis.asyncio import ConnectionPool, Redis
from fastapi import APIRouter
from fastapi.param_functions import Depends

from {{cookiecutter.project_name}}.services.redis.dependency import get_redis_pool
from {{cookiecutter.project_name}}.web.api.redis.schema import RedisValueDTO

router = APIRouter()


@router.get("/", response_model=RedisValueDTO)
async def get_redis_value(
    key: str,
    redis_pool: ConnectionPool = Depends(get_redis_pool),
) -> RedisValueDTO:
    """
    Get value from redis.

    :param key: redis key, to get data from.
    :param redis_pool: redis connection pool.
    :returns: information from redis.
    """
    async with Redis(connection_pool=redis_pool) as redis:
        redis_value = await redis.get(key)
    return RedisValueDTO(
        key=key,
        value=redis_value,
    )


@router.put("/")
async def set_redis_value(
    redis_value: RedisValueDTO,
    redis_pool: ConnectionPool = Depends(get_redis_pool),
) -> None:
    """
    Set value in redis.

    :param redis_value: new value data.
    :param redis_pool: redis connection pool.
    """
    if redis_value.value is not None:
        async with Redis(connection_pool=redis_pool) as redis:
            await redis.set(name=redis_value.key, value=redis_value.value)



import asyncio
from asyncio.events import AbstractEventLoop
import sys
from typing import Any, Generator, AsyncGenerator

import pytest
from fastapi import FastAPI
from httpx import AsyncClient
import uuid
from unittest.mock import Mock

{%- if cookiecutter.enable_redis == "True" %}
from fakeredis import FakeServer
from fakeredis.aioredis import FakeConnection
from redis.asyncio import ConnectionPool
from {{cookiecutter.project_name}}.services.redis.dependency import get_redis_pool
{%- endif %}
{%- if cookiecutter.enable_rmq == "True" %}
from aio_pika import Channel
from aio_pika.abc import AbstractExchange, AbstractQueue
from aio_pika.pool import Pool
from {{cookiecutter.project_name}}.services.rabbit.dependencies import get_rmq_channel_pool
from {{cookiecutter.project_name}}.services.rabbit.lifetime import init_rabbit, shutdown_rabbit

{%- endif %}
{%- if cookiecutter.enable_kafka == "True" %}
from aiokafka import AIOKafkaProducer
from {{cookiecutter.project_name}}.services.kafka.dependencies import get_kafka_producer
from {{cookiecutter.project_name}}.services.kafka.lifetime import init_kafka, shutdown_kafka
{%- endif %}

from {{cookiecutter.project_name}}.settings import settings
from {{cookiecutter.project_name}}.web.application import get_app


{%- if cookiecutter.orm == "sqlalchemy" %}
from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession, AsyncEngine, AsyncConnection
from sqlalchemy.orm import sessionmaker
from {{cookiecutter.project_name}}.db.dependencies import get_db_session
from {{cookiecutter.project_name}}.db.utils import create_database, drop_database
{%- elif cookiecutter.orm == "tortoise" %}
from tortoise.contrib.test import finalizer, initializer
from tortoise import Tortoise
from {{cookiecutter.project_name}}.db.config import MODELS_MODULES, TORTOISE_CONFIG
import nest_asyncio

nest_asyncio.apply()
{%- elif cookiecutter.orm == "ormar" %}
from sqlalchemy.engine import create_engine
from {{cookiecutter.project_name}}.db.config import database
from {{cookiecutter.project_name}}.db.utils import create_database, drop_database
{%- elif cookiecutter.orm == "psycopg" %}
from psycopg import AsyncConnection
from psycopg_pool import AsyncConnectionPool

from {{cookiecutter.project_name}}.db.dependencies import get_db_pool
{%- elif cookiecutter.orm == "piccolo" %}
{%- if cookiecutter.db_info.name == "postgresql" %}
from piccolo.engine.postgres import PostgresEngine
{%- endif %}
from piccolo.table import create_tables, drop_tables

from piccolo.conf.apps import Finder
{%- endif %}


@pytest.fixture(scope="session")
def anyio_backend() -> str:
    """
    Backend for anyio pytest plugin.

    :return: backend name.
    """
    return 'asyncio'

{%- if cookiecutter.orm == "sqlalchemy" %}
@pytest.fixture(scope="session")
async def _engine() -> AsyncGenerator[AsyncEngine, None]:
    """
    Create engine and databases.

    :yield: new engine.
    """
    from {{cookiecutter.project_name}}.db.meta import meta  # noqa: WPS433
    from {{cookiecutter.project_name}}.db.models import load_all_models  # noqa: WPS433

    load_all_models()

    await create_database()

    engine = create_async_engine(str(settings.db_url))
    async with engine.begin() as conn:
        await conn.run_sync(meta.create_all)

    try:
        yield engine
    finally:
        await engine.dispose()
        await drop_database()

@pytest.fixture
async def dbsession(
    _engine: AsyncEngine,
) -> AsyncGenerator[AsyncSession, None]:
    """
    Get session to database.

    Fixture that returns a SQLAlchemy session with a SAVEPOINT, and the rollback to it
    after the test completes.

    :param _engine: current engine.
    :yields: async session.
    """
    connection = await _engine.connect()
    trans = await connection.begin()

    session_maker = sessionmaker(
        connection, expire_on_commit=False, class_=AsyncSession,
    )
    session = session_maker()

    try:
        yield session
    finally:
        await session.close()
        await trans.rollback()
        await connection.close()

{%- elif cookiecutter.orm == "tortoise" %}

@pytest.fixture(autouse=True)
async def initialize_db() -> AsyncGenerator[None, None]:
    """
    Initialize models and database.

    :yields: Nothing.
    """
    initializer(
        MODELS_MODULES,
        db_url=str(settings.db_url),
        app_label="models",
    )
    await Tortoise.init(config=TORTOISE_CONFIG)

    yield

    await Tortoise.close_connections()
    finalizer()

{%- elif cookiecutter.orm == "ormar" %}

@pytest.fixture(autouse=True)
async def initialize_db() -> AsyncGenerator[None, None]:
    """
    Create models and databases.

    :yield: new engine.
    """
    from {{cookiecutter.project_name}}.db.meta import meta  # noqa: WPS433
    from {{cookiecutter.project_name}}.db.models import load_all_models  # noqa: WPS433

    load_all_models()

    create_database()

    engine = create_engine(str(settings.db_url))
    with engine.begin() as conn:
        meta.create_all(conn)

    engine.dispose()

    await database.connect()

    yield

    await database.disconnect()
    drop_database()

{%- elif cookiecutter.orm == "psycopg" %}

async def drop_db() -> None:
    """Drops database after tests."""
    pool = AsyncConnectionPool(conninfo=str(settings.db_url.with_path("/postgres")))
    await pool.wait()
    async with pool.connection() as conn:
        await conn.set_autocommit(True)
        await conn.execute(
            "SELECT pg_terminate_backend(pg_stat_activity.pid) "  # noqa: S608
            "FROM pg_stat_activity "
            "WHERE pg_stat_activity.datname = %(dbname)s "
            "AND pid <> pg_backend_pid();",
            params={
                "dbname": settings.db_base,
            }
        )
        await conn.execute(
            f"DROP DATABASE {settings.db_base}",
        )
    await pool.close()


async def create_db() -> None:  # noqa: WPS217
    """Creates database for tests."""
    pool = AsyncConnectionPool(conninfo=str(settings.db_url.with_path("/postgres")))
    await pool.wait()
    async with pool.connection() as conn_check:
        res = await conn_check.execute(
            "SELECT 1 FROM pg_database WHERE datname=%(dbname)s",
            params={
                "dbname": settings.db_base,
            }
        )
        db_exists = False
        row = await res.fetchone()
        if row is not None:
            db_exists = row[0]

    if db_exists:
        await drop_db()

    async with pool.connection() as conn_create:
        await conn_create.set_autocommit(True)
        await conn_create.execute(
            f"CREATE DATABASE {settings.db_base};",
        )
    await pool.close()


async def create_tables(connection: AsyncConnection[Any]) -> None:
    """
    Create tables for your database.

    Since psycopg doesn't have migration tool,
    you must create your tables for tests.

    :param connection: connection to database.
    """
    {%- if cookiecutter.add_dummy == 'True' %}
    await connection.execute(
        "CREATE TABLE dummy ("
        "id SERIAL primary key,"
        "name VARCHAR(200)"
        ");"
    )
    {%- endif %}
    pass  # noqa: WPS420


@pytest.fixture
async def dbpool() -> AsyncGenerator[AsyncConnectionPool, None]:
    """
    Creates database connections pool to test database.

    This connection must be used in tests and for application.

    :yield: database connections pool.
    """
    await create_db()
    pool = AsyncConnectionPool(conninfo=str(settings.db_url))
    await pool.wait()

    async with pool.connection() as create_conn:
        await create_tables(create_conn)

    try:
        yield pool
    finally:
        await pool.close()
        await drop_db()

{%- elif cookiecutter.orm == "piccolo" %}

{%- if cookiecutter.db_info.name == "postgresql" %}
async def drop_database(engine: PostgresEngine) -> None:
    """
    Drops test database.

    :param engine: engine connected to postgres database.
    """
    await engine.run_ddl(
        "SELECT pg_terminate_backend(pg_stat_activity.pid) "  # noqa: S608
        "FROM pg_stat_activity "
        f"WHERE pg_stat_activity.datname = '{settings.db_base}' "
        "AND pid <> pg_backend_pid();",
    )
    await engine.run_ddl(
        f"DROP DATABASE {settings.db_base};",
    )
{%- endif %}

@pytest.fixture(autouse=True)
async def setup_db() -> AsyncGenerator[None, None]:
    """
    Fixture to create all tables before test and drop them after.

    :yield: nothing.
    """
    {%- if cookiecutter.db_info.name == "postgresql" %}
    engine = PostgresEngine(
        config={
            "database": "postgres",
            "user": settings.db_user,
            "password": settings.db_pass,
            "host": settings.db_host,
            "port": settings.db_port,
        },
    )
    await engine.start_connection_pool()

    db_exists = await engine.run_ddl(
        f"SELECT 1 FROM pg_database WHERE datname='{settings.db_base}'"  # noqa: S608
    )
    if db_exists:
        await drop_database(engine)
    await engine.run_ddl(f"CREATE DATABASE {settings.db_base}")
    {%- endif %}
    tables = Finder().get_table_classes()
    create_tables(*tables, if_not_exists=True)

    yield

    drop_tables(*tables)
    {%- if cookiecutter.db_info.name == "postgresql" %}
    await drop_database(engine)
    {%- endif %}

{%- endif %}

{%- if cookiecutter.enable_rmq == 'True' %}

@pytest.fixture
async def test_rmq_pool() -> AsyncGenerator[Channel, None]:
    """
    Create rabbitMQ pool.

    :yield: channel pool.
    """
    app_mock = Mock()
    init_rabbit(app_mock)
    yield app_mock.state.rmq_channel_pool
    await shutdown_rabbit(app_mock)


@pytest.fixture
async def test_exchange_name() -> str:
    """
    Name of an exchange to use in tests.

    :return: name of an exchange.
    """
    return uuid.uuid4().hex


@pytest.fixture
async def test_routing_key() -> str:
    """
    Name of routing key to use whild binding test queue.

    :return: key string.
    """
    return uuid.uuid4().hex


@pytest.fixture
async def test_exchange(
    test_exchange_name: str,
    test_rmq_pool: Pool[Channel],
) -> AsyncGenerator[AbstractExchange, None]:
    """
    Creates test exchange.

    :param test_exchange_name: name of an exchange to create.
    :param test_rmq_pool: channel pool for rabbitmq.
    :yield: created exchange.
    """
    async with test_rmq_pool.acquire() as conn:
        exchange = await conn.declare_exchange(
            name=test_exchange_name,
            auto_delete=True,
        )
        yield exchange

        await exchange.delete(if_unused=False)


@pytest.fixture
async def test_queue(
    test_exchange: AbstractExchange,
    test_rmq_pool: Pool[Channel],
    test_routing_key: str,
) -> AsyncGenerator[AbstractQueue, None]:
    """
    Creates queue connected to exchange.

    :param test_exchange: exchange to bind queue to.
    :param test_rmq_pool: channel pool for rabbitmq.
    :param test_routing_key: routing key to use while binding.
    :yield: queue binded to test exchange.
    """
    async with test_rmq_pool.acquire() as conn:
        queue = await conn.declare_queue(name=uuid.uuid4().hex)
        await queue.bind(
            exchange=test_exchange,
            routing_key=test_routing_key,
        )
        yield queue

        await queue.delete(if_unused=False, if_empty=False)

{%- endif %}

{%- if cookiecutter.enable_kafka == "True" %}

@pytest.fixture
async def test_kafka_producer() -> AsyncGenerator[AIOKafkaProducer, None]:
    """
    Creates kafka's producer.

    :yields: kafka's producer.
    """
    app_mock = Mock()
    await init_kafka(app_mock)
    yield app_mock.state.kafka_producer
    await shutdown_kafka(app_mock)

{%- endif %}

{% if cookiecutter.enable_redis == "True" -%}
@pytest.fixture
async def fake_redis_pool() -> AsyncGenerator[ConnectionPool, None]:
    """
    Get instance of a fake redis.

    :yield: FakeRedis instance.
    """
    server = FakeServer()
    server.connected = True
    pool = ConnectionPool(connection_class=FakeConnection, server=server)

    yield pool

    await pool.disconnect()

{%- endif %}

@pytest.fixture
def fastapi_app(
    {%- if cookiecutter.orm == "sqlalchemy" %}
    dbsession: AsyncSession,
    {%- elif cookiecutter.orm == "psycopg" %}
    dbpool: AsyncConnectionPool,
    {%- endif %}
    {% if cookiecutter.enable_redis == "True" -%}
    fake_redis_pool: ConnectionPool,
    {%- endif %}
    {%- if cookiecutter.enable_rmq == 'True' %}
    test_rmq_pool: Pool[Channel],
    {%- endif %}
    {%- if cookiecutter.enable_kafka == "True" %}
    test_kafka_producer: AIOKafkaProducer,
    {%- endif %}
) -> FastAPI:
    """
    Fixture for creating FastAPI app.

    :return: fastapi app with mocked dependencies.
    """
    application = get_app()
    {%- if cookiecutter.orm == "sqlalchemy" %}
    application.dependency_overrides[get_db_session] = lambda: dbsession
    {%- elif cookiecutter.orm == "psycopg" %}
    application.dependency_overrides[get_db_pool] = lambda: dbpool
    {%- endif %}
    {%- if cookiecutter.enable_redis == "True" %}
    application.dependency_overrides[get_redis_pool] = lambda: fake_redis_pool
    {%- endif %}
    {%- if cookiecutter.enable_rmq == 'True' %}
    application.dependency_overrides[get_rmq_channel_pool] = lambda: test_rmq_pool
    {%- endif %}
    {%- if cookiecutter.enable_kafka == "True" %}
    application.dependency_overrides[get_kafka_producer] = lambda: test_kafka_producer
    {%- endif %}
    return application  # noqa: WPS331


@pytest.fixture
async def client(
    fastapi_app: FastAPI,
    anyio_backend: Any
) -> AsyncGenerator[AsyncClient, None]:
    """
    Fixture that creates client for requesting server.

    :param fastapi_app: the application.
    :yield: client for the app.
    """
    async with AsyncClient(app=fastapi_app, base_url="http://test") as ac:
            yield ac



from fastapi import FastAPI
from redis.asyncio import ConnectionPool
from {{cookiecutter.project_name}}.settings import settings


def init_redis(app: FastAPI) -> None:  # pragma: no cover
    """
    Creates connection pool for redis.

    :param app: current fastapi application.
    """
    app.state.redis_pool = ConnectionPool.from_url(
        str(settings.redis_url),
    )


async def shutdown_redis(app: FastAPI) -> None:  # pragma: no cover
    """
    Closes redis connection pool.

    :param app: current FastAPI app.
    """
    await app.state.redis_pool.disconnect()



import os
import subprocess
from typing import Optional
from fastapi_template.input_model import BuilderContext
from fastapi_template.__main__ import generate_project


def generate_project_and_chdir(context: BuilderContext):
    generate_project(context)
    os.chdir(context.project_name)


def run_pre_commit() -> int:
    results = subprocess.run(["pre-commit", "run", "-a"])
    return results.returncode


def run_docker_compose_command(
    command: Optional[str] = None,
) -> subprocess.CompletedProcess:
    docker_command = [
        "docker-compose",
        "-f",
        "deploy/docker-compose.yml",
        "--project-directory",
        ".",
    ]
    if command:
        docker_command.extend(command.split())
    else:
        docker_command.extend(
            [
                "build",
            ]
        )
    return subprocess.run(docker_command)


def run_default_check(context: BuilderContext, without_pytest=False):
    generate_project_and_chdir(context)
    assert run_pre_commit() == 0

    if without_pytest:
        return

    build = run_docker_compose_command("build")
    assert build.returncode == 0
    tests = run_docker_compose_command("run --rm api pytest -vv .")
    assert tests.returncode == 0



from typing import Optional

import strawberry


@strawberry.type
class RedisDTO:
    """Output type for redis queries."""

    key: str
    value: Optional[str]


@strawberry.input
class RedisDTOInput:
    """Input type for redis mutation."""

    key: str
    value: str



import ormar

from {{cookiecutter.project_name}}.db.base import BaseMeta


class DummyModel(ormar.Model):
    """Model for demo purpose."""

    class Meta(BaseMeta):
        tablename = "dummy_model"

    id: int = ormar.Integer(primary_key=True)
    name: str = ormar.String(max_length=200)  # noqa: WPS432



"""Initial migration.

Revision ID: 819cbf6e030b
Revises:
Create Date: 2021-08-16 16:53:05.484024

"""


# revision identifiers, used by Alembic.
revision = "819cbf6e030b"
down_revision = None
branch_labels = None
depends_on = None


def upgrade() -> None:
    pass


def downgrade() -> None:
    pass



#!/usr/bin/env python
import json
import os
import shutil
import subprocess

from termcolor import cprint, colored
from pathlib import Path

CONDITIONAL_MANIFEST = "conditional_files.json"
REPLACE_MANIFEST = "replaceable_files.json"


def delete_resource(resource):
    if os.path.isfile(resource):
        os.remove(resource)
    elif os.path.isdir(resource):
        shutil.rmtree(resource)


def delete_resources_for_disabled_features():
    with open(CONDITIONAL_MANIFEST) as manifest_file:
        manifest = json.load(manifest_file)
        for feature_name, feature in manifest.items():
            if feature["enabled"].lower() != "true":
                text = "{} resources for disabled feature {}...".format(
                    colored("Removing", color="red"),
                    colored(feature_name, color="magenta", attrs=["underline"]),
                )
                print(text)
                for resource in feature["resources"]:
                    delete_resource(resource)
    delete_resource(CONDITIONAL_MANIFEST)
    cprint("cleanup complete!", color="green")


def replace_resources():
    print(
        "⭐ Placing {} nicely in your {} ⭐".format(
            colored("resources", color="green"), colored("new project", color="blue")
        )
    )
    with open(REPLACE_MANIFEST) as replace_manifest:
        manifest = json.load(replace_manifest)
        for target, replaces in manifest.items():
            target_path = Path(target)
            delete_resource(target_path)
            for src_file in map(Path, replaces):
                if src_file.exists():
                    shutil.move(src_file, target_path)
    delete_resource(REPLACE_MANIFEST)
    print(
        "Resources are happy to be where {}.".format(
            colored("they are needed the most", color="green", attrs=["underline"])
        )
    )


def init_repo():
    subprocess.run(["git", "init"], stdout=subprocess.PIPE)
    cprint("Git repository initialized.", "green")
    subprocess.run(["git", "add", "."], stdout=subprocess.PIPE)
    cprint("Added files to index.", "green")
    subprocess.run(["poetry", "install", "-n"])
    subprocess.run(["poetry", "run", "pre-commit", "install"])
    cprint("pre-commit installed.", "green")
    subprocess.run(["poetry", "run", "pre-commit", "run", "-a"])
    subprocess.run(["git", "add", "."], stdout=subprocess.PIPE)
    subprocess.run(["git", "commit", "-m", "Initial commit"], stdout=subprocess.PIPE)

if __name__ == "__main__":
    delete_resources_for_disabled_features()
    replace_resources()
    init_repo()



import strawberry
from strawberry.types import Info

from {{cookiecutter.project_name}}.web.gql.context import Context
from {{cookiecutter.project_name}}.web.gql.redis.schema import RedisDTO, RedisDTOInput
from redis.asyncio import Redis

@strawberry.type
class Mutation:
    """Mutations for redis."""

    @strawberry.mutation(description="Set value in redis")
    async def set_redis_value(
        self,
        data: RedisDTOInput,
        info: Info[Context, None],
    ) -> RedisDTO:
        """
        Sets value in redis.

        :param data: key and value to insert.
        :param info: connection info.
        :return: key and value.
        """
        async with Redis(connection_pool=info.context.redis_pool) as redis:
            await redis.set(name=data.key, value=data.value)
        return RedisDTO(key=data.key, value=data.value)  # type: ignore



from termios import OFDEL
from {{cookiecutter.project_name}}.db.models.dummy_model import DummyModel
from typing import Any

from fastapi import Depends
from psycopg_pool import AsyncConnectionPool
from psycopg.rows import class_row
from {{cookiecutter.project_name}}.db.dependencies import get_db_pool
from typing import List, Optional

class DummyDAO:
    """Class for accessing dummy table."""

    def __init__(
        self,
        db_pool: AsyncConnectionPool = Depends(get_db_pool),
    ):
        self.db_pool = db_pool


    async def create_dummy_model(self, name: str) -> None:
        """
        Creates new dummy in a database.

        :param name: name of a dummy.
        """
        async with self.db_pool.connection() as connection:
            async with connection.cursor(binary=True) as cur:
                await cur.execute(
                    "INSERT INTO dummy (name) VALUES (%(name)s);",
                    params={
                        "name": name,
                    }
                )

    async def get_all_dummies(self, limit: int, offset: int) -> List[DummyModel]:
        """
        Get all dummy models with limit/offset pagination.

        :param limit: limit of dummies.
        :param offset: offset of dummies.
        :return: stream of dummies.
        """
        async with self.db_pool.connection() as connection:
            async with connection.cursor(
                binary=True,
                row_factory=class_row(DummyModel)
            ) as cur:
                res = await cur.execute(
                    "SELECT id, name FROM dummy LIMIT %(limit)s OFFSET %(offset)s;",
                    params={
                        "limit": limit,
                        "offset": offset,
                    }
                )
                return await res.fetchall()

    async def filter(
        self,
        name: Optional[str] = None,
    ) -> List[DummyModel]:
        """
        Get specific dummy model.

        :param name: name of dummy instance.
        :return: dummy models.
        """
        async with self.db_pool.connection() as connection:
            async with connection.cursor(
                binary=True,
                row_factory=class_row(DummyModel)
            ) as cur:
                if name is not None:
                    res = await cur.execute(
                        "SELECT id, name FROM dummy WHERE name=%(name)s;",
                        params={
                            "name": name,
                        }
                    )
                else:
                    res = await cur.execute("SELECT id, name FROM dummy;")
                return await res.fetchall()



"""Package for dummy model."""

from {{cookiecutter.project_name}}.web.gql.dummy.mutation import Mutation
from {{cookiecutter.project_name}}.web.gql.dummy.query import Query

__all__ = ["Query", "Mutation"]



from ormar import ModelMeta

from {{cookiecutter.project_name}}.db.config import database
from {{cookiecutter.project_name}}.db.meta import meta


class BaseMeta(ModelMeta):
    """Base metadata for models."""

    database = database
    metadata = meta



import strawberry
from strawberry.types import Info

from {{cookiecutter.project_name}}.web.gql.context import Context
from {{cookiecutter.project_name}}.web.gql.kafka.schema import KafkaMessageDTO


@strawberry.type
class Mutation:
    """Mutation for rabbit package."""

    @strawberry.mutation(description="Send message to Kafka")
    async def send_kafka_message(
        self, message: KafkaMessageDTO,
        info: Info[Context, None],
    ) -> None:
        """
        Sends a message in Kafka.

        :param message: message to publish.
        :param info: current context.
        """
        await info.context.kafka_producer.send(
            topic=message.topic,
            value=message.message.encode(),
        )




import json
from pathlib import Path

from cookiecutter.exceptions import FailedHookException, OutputDirExistsException
from cookiecutter.main import cookiecutter
from termcolor import cprint

from fastapi_template.cli import run_command
from fastapi_template.input_model import BuilderContext

script_dir = Path(__file__).parent


def generate_project(context: BuilderContext) -> None:
    """
    Generate actual project with given context.

    :param context: builder_context
    """
    try:
        cookiecutter(
            template=f"{script_dir}/template",
            extra_context=context.dict(),
            default_config=BuilderContext().dict(),
            no_input=True,
            overwrite_if_exists=context.force,
        )
    except (FailedHookException, OutputDirExistsException) as exc:
        if isinstance(exc, OutputDirExistsException):
            cprint("Directory with such name already exists!", "red")
        return
    cprint(
        "Project successfully generated. You can read information about usage in README.md"
    )


def main() -> None:
    """Starting point."""
    run_command(generate_project)


if __name__ == "__main__":
    main()



import asyncio
import uuid

import pytest
from aiokafka import AIOKafkaConsumer
from fastapi import FastAPI
from httpx import AsyncClient
from starlette import status

from {{cookiecutter.project_name}}.settings import settings


@pytest.mark.anyio
async def test_message_publishing(
    fastapi_app: FastAPI,
    client: AsyncClient,
) -> None:
    """
    Test that messages are published correctly.

    It sends message to kafka, reads it and
    validates that received message has the same
    value.

    :param fastapi_app: current application.
    :param client: httpx client.
    """
    topic_name = uuid.uuid4().hex
    message = uuid.uuid4().hex
    consumer = AIOKafkaConsumer(
        topic_name,
        bootstrap_servers=settings.kafka_bootstrap_servers,
    )
    await consumer.start()

    {%- if cookiecutter.api_type == 'rest' %}
    url = fastapi_app.url_path_for("send_kafka_message")
    response = await client.post(
        url,
        json={
            "topic": topic_name,
            "message": message,
        },
    )
    {%- elif cookiecutter.api_type == 'graphql' %}
    url = fastapi_app.url_path_for('handle_http_post')
    response = await client.post(
        url,
        json={
            "query": "mutation($message:KafkaMessageDTO!)"
                     "{sendKafkaMessage(message:$message)}",
            "variables": {
                "message": {
                    "topic": topic_name,
                    "message": message,
                },
            },
        },
    )
    {%- endif %}


    assert response.status_code == status.HTTP_200_OK

    msg = await asyncio.wait_for(consumer.getone(), timeout=1)
    assert msg.value == message.encode()



import enum
from typing import Optional, Callable, Any

from pydantic import BaseModel
import click
import abc
from simple_term_menu import TerminalMenu
from collections import UserDict


class Database(BaseModel):
    name: str
    image: Optional[str]
    driver: Optional[str]
    async_driver: Optional[str]
    port: Optional[int]
    driver_short: Optional[str]


class MenuEntry(BaseModel):
    code: str
    cli_name: Optional[str]
    user_view: str
    description: str
    is_hidden: Optional[Callable[["BuilderContext"], bool]]
    additional_info: Any

    @property
    def generated_name(self) -> str:
        """
        Property to generate parameter name.

        It checks if cli_name is present,
        otherwise, code is used.

        :return: string to use in CLI.
        """
        if self.cli_name:
            return self.cli_name
        return self.code


SKIP_ENTRY = MenuEntry(
    code="skip",
    user_view="skip",
    description="skip",
)


class BaseMenuModel(BaseModel, abc.ABC):
    title: str
    entries: list[MenuEntry]

    def _preview(self, current_value: str):

        for entry in self.entries:
            if entry.user_view == current_value:
                return entry.description
        return "Unknown value"

    @abc.abstractmethod
    def get_cli_options(self) -> list[click.Option]:
        pass

    @abc.abstractmethod
    def ask(self, context: "BuilderContext") -> Optional["BuilderContext"]:
        pass

    @abc.abstractmethod
    def need_ask(self, context: "BuilderContext") -> bool:
        pass

    def after_ask(self, context: "BuilderContext") -> "BuilderContext":
        """Function run after the menu finished work."""
        return context


class SingularMenuModel(BaseMenuModel):
    code: str
    cli_name: Optional[str]
    description: str
    before_ask_fun: Optional[Callable[["BuilderContext"], Optional[MenuEntry]]]
    after_ask_fun: Optional[
        Callable[["BuilderContext", "SingularMenuModel"], "BuilderContext"]
    ]
    parser: Optional[Callable[[str], Any]]

    def get_cli_options(self) -> list[click.Option]:
        cli_name = self.code
        if self.cli_name is not None:
            cli_name = self.cli_name
        choices = [entry.generated_name for entry in self.entries]
        return [
            click.Option(
                param_decls=[f"--{cli_name}", self.code],
                type=click.Choice(choices, case_sensitive=False),
                default=None,
                help=self.description,
            )
        ]

    def need_ask(self, context: "BuilderContext") -> bool:
        if getattr(context, self.code, None) is None:
            return True
        return False

    def ask(self, context: "BuilderContext") -> Optional["BuilderContext"]:
        chosen_entry = None
        if self.before_ask_fun is not None:
            chosen_entry = self.before_ask_fun(context)

        ctx_value = context.dict().get(self.code)
        if ctx_value:
            for entry in self.entries:
                if entry.code == ctx_value:
                    chosen_entry = entry

        if not chosen_entry:
            available_entries = []
            for entry in self.entries:
                if entry.is_hidden is None:
                    available_entries.append(entry)
                elif not entry.is_hidden(context):
                    available_entries.append(entry)

            menu = TerminalMenu(
                title=self.title,
                menu_entries=[entry.user_view for entry in available_entries],
                multi_select=False,
                preview_title="Description",
                preview_command=self._preview,
                preview_size=0.5,
            )
            idx = menu.show()
            if idx is None:
                return None

            chosen_entry = available_entries[idx]

        if chosen_entry == SKIP_ENTRY:
            return

        setattr(context, self.code, chosen_entry.code)

        return context

    def after_ask(self, context: "BuilderContext") -> "BuilderContext":
        if self.after_ask_fun:
            return self.after_ask_fun(context, self)
        return super().after_ask(context)


class MultiselectMenuModel(BaseMenuModel):
    before_ask: Optional[Callable[["BuilderContext"], Optional[list[MenuEntry]]]]

    def get_cli_options(self) -> list[click.Option]:
        options = []
        for entry in self.entries:
            options.append(
                click.Option(
                    param_decls=[f"--{entry.generated_name}", entry.code],
                    is_flag=True,
                    help=entry.user_view,
                    default=None,
                )
            )
        return options

    def need_ask(self, context: "BuilderContext") -> bool:
        for entry in self.entries:
            if getattr(context, entry.code, None) is None:
                return True
        return False

    def ask(self, context: "BuilderContext") -> Optional["BuilderContext"]:
        chosen_entries = None
        if self.before_ask is not None:
            chosen_entries = self.before_ask(context)

        if chosen_entries is None:
            unknown_entries = []
            for entry in self.entries:
                if not context.dict().get(entry.code):
                    unknown_entries.append(entry)

            visible_entries = []
            for entry in unknown_entries:
                if entry.is_hidden is None:
                    visible_entries.append(entry)
                elif not entry.is_hidden(context):
                    visible_entries.append(entry)

            menu = TerminalMenu(
                title=self.title,
                menu_entries=[entry.user_view for entry in visible_entries],
                multi_select=True,
                preview_title="Description",
                preview_command=self._preview,
            )

            idxs = menu.show()

            if idxs is None:
                return None

            chosen_entries = []
            for idx in idxs:
                chosen_entries.append(visible_entries[idx])

        if chosen_entries == [SKIP_ENTRY]:
            return context

        for entry in chosen_entries:
            setattr(context, entry.code, True)

        return context


class BuilderContext(UserDict):
    """Options for project generation."""

    def __init__(self, **kwargs: Any) -> None:
        self.__dict__["data"] = kwargs

    def __getattr__(self, name: str) -> Any:
        try:
            return self.__dict__["data"][name]
        except KeyError:
            cls_name = self.__class__.__name__
            raise AttributeError(f"'{cls_name}' object has no attribute '{name}'")

    def __setattr__(self, name: str, value: Any) -> None:
        self[name] = value

    def dict(self) -> str:
        return self.__dict__["data"]



import aio_pika
from aio_pika.abc import AbstractChannel, AbstractRobustConnection
from aio_pika.pool import Pool
from fastapi import FastAPI

from {{cookiecutter.project_name}}.settings import settings


def init_rabbit(app: FastAPI) -> None:  # pragma: no cover
    """
    Initialize rabbitmq pools.

    :param app: current FastAPI application.
    """

    async def get_connection() -> AbstractRobustConnection:  # noqa: WPS430
        """
        Creates connection to RabbitMQ using url from settings.

        :return: async connection to RabbitMQ.
        """
        return await aio_pika.connect_robust(str(settings.rabbit_url))

    # This pool is used to open connections.
    connection_pool: Pool[AbstractRobustConnection] = Pool(
        get_connection,
        max_size=settings.rabbit_pool_size,
    )

    async def get_channel() -> AbstractChannel:  # noqa: WPS430
        """
        Open channel on connection.

        Channels are used to actually communicate with rabbitmq.

        :return: connected channel.
        """
        async with connection_pool.acquire() as connection:
            return await connection.channel()

    # This pool is used to open channels.
    channel_pool: Pool[aio_pika.Channel] = Pool(
        get_channel,
        max_size=settings.rabbit_channel_pool_size,
    )

    app.state.rmq_pool = connection_pool
    app.state.rmq_channel_pool = channel_pool


async def shutdown_rabbit(app: FastAPI) -> None:  # pragma: no cover
    """
    Close all connection and pools.

    :param app: current application.
    """
    await app.state.rmq_channel_pool.close()
    await app.state.rmq_pool.close()



import logging
import sys
from typing import Any, Union

from loguru import logger

from {{cookiecutter.project_name}}.settings import settings

{%- if cookiecutter.otlp_enabled == "True" %}
from opentelemetry.trace import INVALID_SPAN, INVALID_SPAN_CONTEXT, get_current_span
{%- endif %}


class InterceptHandler(logging.Handler):
    """
    Default handler from examples in loguru documentation.

    This handler intercepts all log requests and
    passes them to loguru.

    For more info see:
    https://loguru.readthedocs.io/en/stable/overview.html#entirely-compatible-with-standard-logging
    """

    def emit(self, record: logging.LogRecord) -> None:  # pragma: no cover
        """
        Propagates logs to loguru.

        :param record: record to log.
        """
        try:
            level: Union[str, int] = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        # Find caller from where originated the logged message
        frame, depth = logging.currentframe(), 2
        while frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back  # type: ignore
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(
            level,
            record.getMessage(),
        )

{%- if cookiecutter.otlp_enabled == "True" %}

def record_formatter(record: dict[str, Any]) -> str:  # pragma: no cover
    """
    Formats the record.

    This function formats message
    by adding extra trace information to the record.

    :param record: record information.
    :return: format string.
    """
    log_format = (
        "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> "
        "| <level>{level: <8}</level> "
        "| <magenta>trace_id={extra[trace_id]}</magenta> "
        "| <blue>span_id={extra[span_id]}</blue> "
        "| <cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> "
        "- <level>{message}</level>\n"
    )

    span = get_current_span()
    record["extra"]["span_id"] = 0
    record["extra"]["trace_id"] = 0
    if span != INVALID_SPAN:
        span_context = span.get_span_context()
        if span_context != INVALID_SPAN_CONTEXT:
            record["extra"]["span_id"] = format(span_context.span_id, "016x")
            record["extra"]["trace_id"] = format(span_context.trace_id, "032x")

    if record["exception"]:
        log_format = f"{log_format}{{'{{'}}exception{{'}}'}}"

    return log_format

{%- endif %}

def configure_logging() -> None:  # pragma: no cover
    """Configures logging."""
    loggers = (
        logging.getLogger(name)
        for name in logging.root.manager.loggerDict
        if name.startswith("uvicorn.")
    )
    for uvicorn_logger in loggers:
        uvicorn_logger.handlers = []

    # change handler for default uvicorn logger
    intercept_handler = InterceptHandler()
    logging.getLogger("uvicorn").handlers = [intercept_handler]
    logging.getLogger("uvicorn.access").handlers = [intercept_handler]

    # set logs output, level and format
    logger.remove()
    logger.add(
        sys.stdout,
        level=settings.log_level.value,
        {%- if cookiecutter.otlp_enabled == "True" %}
        format=record_formatter,  # type: ignore
        {%- endif %}
    )



from sqlalchemy.sql.schema import Column
from sqlalchemy.sql.sqltypes import Integer, String

from {{cookiecutter.project_name}}.db.base import Base


class DummyModel(Base):
    """Model for demo purpose."""

    __tablename__ = "dummy_model"

    id = Column(Integer(), primary_key=True, autoincrement=True)
    name = Column(String(length=200))  # noqa: WPS432



from typing import List, Optional
from fastapi import Depends
from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession

from {{cookiecutter.project_name}}.db.dependencies import get_db_session
from {{cookiecutter.project_name}}.db.models.dummy_model import DummyModel


class DummyDAO:
    """Class for accessing dummy table."""

    def __init__(self, session: AsyncSession = Depends(get_db_session)):
        self.session = session

    async def create_dummy_model(self, name: str) -> None:
        """
        Add single dummy to session.

        :param name: name of a dummy.
        """
        self.session.add(DummyModel(name=name))

    async def get_all_dummies(self, limit: int, offset: int) -> List[DummyModel]:
        """
        Get all dummy models with limit/offset pagination.

        :param limit: limit of dummies.
        :param offset: offset of dummies.
        :return: stream of dummies.
        """
        raw_dummies = await self.session.execute(
            select(DummyModel).limit(limit).offset(offset),
        )

        return raw_dummies.scalars().fetchall()

    async def filter(
        self,
        name: Optional[str] = None
    ) -> List[DummyModel]:
        """
        Get specific dummy model.

        :param name: name of dummy instance.
        :return: dummy models.
        """
        query = select(DummyModel)
        if name:
            query = query.where(DummyModel.name == name)
        rows = await self.session.execute(query)
        return rows.scalars().fetchall()



import uuid
import pytest
from httpx import AsyncClient
from fastapi import FastAPI
from typing import Any
{%- if cookiecutter.orm == 'sqlalchemy' %}
from sqlalchemy.ext.asyncio import AsyncSession
{%- elif cookiecutter.orm == 'psycopg' %}
from psycopg.connection_async import AsyncConnection
from psycopg_pool import AsyncConnectionPool
{%- endif %}
from starlette import status
from {{cookiecutter.project_name}}.db.models.dummy_model import DummyModel
from {{cookiecutter.project_name}}.db.dao.dummy_dao import DummyDAO

@pytest.mark.anyio
async def test_creation(
    fastapi_app: FastAPI,
    client: AsyncClient,
    {%- if cookiecutter.orm == "sqlalchemy" %}
    dbsession: AsyncSession,
    {%- elif cookiecutter.orm == "psycopg" %}
    dbpool: AsyncConnectionPool,
    {%- endif %}
) -> None:
    """Tests dummy instance creation."""
    {%- if cookiecutter.api_type == 'rest' %}
    url = fastapi_app.url_path_for('create_dummy_model')
    {%- elif cookiecutter.api_type == 'graphql' %}
    url = fastapi_app.url_path_for('handle_http_post')
    {%- endif %}
    test_name = uuid.uuid4().hex
    {%- if cookiecutter.api_type == 'rest' %}
    response = await client.put(url, json={
        "name": test_name
    })
    {%- elif cookiecutter.api_type == 'graphql' %}
    response = await client.post(
        url,
        json={
            "query": "mutation($name: String!){createDummyModel(name: $name)}",
            "variables": {"name": test_name},
        },
    )
    {%- endif %}
    assert response.status_code == status.HTTP_200_OK
    {%- if cookiecutter.orm == "sqlalchemy" %}
    dao = DummyDAO(dbsession)
    {%- elif cookiecutter.orm == "psycopg" %}
    dao = DummyDAO(dbpool)
    {%- elif cookiecutter.orm in ["tortoise", "ormar", "piccolo"] %}
    dao = DummyDAO()
    {%- endif %}
    instances = await dao.filter(name=test_name)
    assert instances[0].name == test_name


@pytest.mark.anyio
async def test_getting(
    fastapi_app: FastAPI,
    client: AsyncClient,
    {%- if cookiecutter.orm == "sqlalchemy" %}
    dbsession: AsyncSession,
    {%- elif cookiecutter.orm == "psycopg" %}
    dbpool: AsyncConnectionPool,
    {%- endif %}
) -> None:
    """Tests dummy instance retrieval."""
    {%- if cookiecutter.orm == "sqlalchemy" %}
    dao = DummyDAO(dbsession)
    {%- elif cookiecutter.orm == "psycopg" %}
    dao = DummyDAO(dbpool)
    {%- elif cookiecutter.orm in ["tortoise", "ormar", "piccolo"] %}
    dao = DummyDAO()
    {%- endif %}
    test_name = uuid.uuid4().hex
    await dao.create_dummy_model(name=test_name)

    {%- if cookiecutter.api_type == 'rest' %}
    url = fastapi_app.url_path_for('get_dummy_models')
    {%- elif cookiecutter.api_type == 'graphql' %}
    url = fastapi_app.url_path_for('handle_http_post')
    {%- endif %}

    {%- if cookiecutter.api_type == 'rest' %}
    response = await client.get(url)
    dummies = response.json()
    {%- elif cookiecutter.api_type == 'graphql' %}
    response = await client.post(
        url,
        json={"query": "query{dumies:getDummyModels{id name}}"},
    )
    dummies = response.json()["data"]["dumies"]
    {%- endif %}

    assert response.status_code == status.HTTP_200_OK
    assert len(dummies) == 1
    assert dummies[0]['name'] == test_name



from aio_pika import Channel, Message
from aio_pika.pool import Pool
from fastapi import APIRouter, Depends

from {{cookiecutter.project_name}}.services.rabbit.dependencies import get_rmq_channel_pool
from {{cookiecutter.project_name}}.web.api.rabbit.schema import RMQMessageDTO

router = APIRouter()


@router.post("/")
async def send_rabbit_message(
    message: RMQMessageDTO,
    pool: Pool[Channel] = Depends(get_rmq_channel_pool),
) -> None:
    """
    Posts a message in a rabbitMQ's exchange.

    :param message: message to publish to rabbitmq.
    :param pool: rabbitmq channel pool
    """
    async with pool.acquire() as conn:
        exchange = await conn.declare_exchange(
            name=message.exchange_name,
            auto_delete=True,
        )
        await exchange.publish(
            message=Message(
                body=message.message.encode("utf-8"),
                content_encoding="utf-8",
                content_type="text/plain",
            ),
            routing_key=message.routing_key,
        )



from fastapi import APIRouter

from {{cookiecutter.project_name}}.web.api.echo.schema import Message

router = APIRouter()


@router.post("/", response_model=Message)
async def send_echo_message(
    incoming_message: Message,
) -> Message:
    """
    Sends echo back to user.

    :param incoming_message: incoming message.
    :returns: message same as the incoming.
    """
    return incoming_message



from {{cookiecutter.project_name}}.db.models.dummy_model import DummyModel
from typing import List, Optional


class DummyDAO:
    """Class for accessing dummy table."""

    async def create_dummy_model(self, name: str) -> None:
        """
        Add single dummy to session.

        :param name: name of a dummy.
        """
        await DummyModel.create(name=name)

    async def get_all_dummies(self, limit: int, offset: int) -> List[DummyModel]:
        """
        Get all dummy models with limit/offset pagination.

        :param limit: limit of dummies.
        :param offset: offset of dummies.
        :return: stream of dummies.
        """
        return (
            await DummyModel.all().offset(offset).limit(limit)
        )

    async def filter(self, name: Optional[str] = None) -> List[DummyModel]:
        """
        Get specific dummy model.

        :param name: name of dummy instance.
        :return: dummy models.
        """
        query = DummyModel.all()
        if name:
            query = query.filter(name=name)
        return await query



from psycopg_pool import AsyncConnectionPool
from starlette.requests import Request


async def get_db_pool(request: Request) -> AsyncConnectionPool:
    """
    Return database connections pool.

    :param request: current request.
    :returns: database connections pool.
    """
    return request.app.state.db_pool



from typing import AsyncGenerator

from redis.asyncio import Redis
from starlette.requests import Request


async def get_redis_pool(request: Request) -> AsyncGenerator[Redis, None]:  # pragma: no cover
    """
    Returns connection pool.

    You can use it like this:

    >>> from redis.asyncio import ConnectionPool, Redis
    >>>
    >>> async def handler(redis_pool: ConnectionPool = Depends(get_redis_pool)):
    >>>     async with Redis(connection_pool=redis_pool) as redis:
    >>>         await redis.get('key')

    I use pools so you don't acquire connection till the end of the handler.

    :param request: current request.
    :returns:  redis connection pool.
    """
    return request.app.state.redis_pool



from typing import List

from fastapi import APIRouter
from fastapi.param_functions import Depends

from {{cookiecutter.project_name}}.db.dao.dummy_dao import DummyDAO
from {{cookiecutter.project_name}}.db.models.dummy_model import DummyModel
from {{cookiecutter.project_name}}.web.api.dummy.schema import DummyModelDTO, DummyModelInputDTO

router = APIRouter()


@router.get("/", response_model=List[DummyModelDTO])
async def get_dummy_models(
    limit: int = 10,
    offset: int = 0,
    dummy_dao: DummyDAO = Depends(),
) -> List[DummyModel]:
    """
    Retrieve all dummy objects from the database.

    :param limit: limit of dummy objects, defaults to 10.
    :param offset: offset of dummy objects, defaults to 0.
    :param dummy_dao: DAO for dummy models.
    :return: list of dummy obbjects from database.
    """
    return await dummy_dao.get_all_dummies(limit=limit, offset=offset)


@router.put("/")
async def create_dummy_model(
    new_dummy_object: DummyModelInputDTO,
    dummy_dao: DummyDAO = Depends(),
) -> None:
    """
    Creates dummy model in the database.

    :param new_dummy_object: new dummy model item.
    :param dummy_dao: DAO for dummy models.
    """
    await dummy_dao.create_dummy_model(**new_dummy_object.dict())



"""Created Dummy Model.

Revision ID: 2b7380507a71
Revises: 819cbf6e030b
Create Date: 2021-08-16 16:55:25.157309

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "2b7380507a71"
down_revision = "819cbf6e030b"
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "dummy_model",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=200), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("dummy_model")
    # ### end Alembic commands ###



import strawberry

from {{cookiecutter.project_name}}.db.dao.dummy_dao import DummyDAO
from {{cookiecutter.project_name}}.web.gql.context import Context

from strawberry.types import Info

@strawberry.type
class Mutation:
    """Mutations for dummies."""

    @strawberry.mutation(description="Create dummy object in a database")
    async def create_dummy_model(
        self,
        {%- if cookiecutter.orm in ["sqlalchemy", "psycopg"] %}
        info: Info[Context, None],
        {%- endif %}
        name: str,
    ) -> str:
        """
        Creates dummy model in a database.

        {% if cookiecutter.orm in ["sqlalchemy", "psycopg"] -%}
        :param info: connection info.
        {% endif -%}
        :param name: name of a dummy.
        :return: name of a dummt model.
        """
        {%- if cookiecutter.orm == "sqlalchemy" %}
        dao = DummyDAO(info.context.db_connection)
        {%- elif cookiecutter.orm == "psycopg" %}
        dao = DummyDAO(info.context.db_pool)
        {%- else %}
        dao = DummyDAO()
        {%- endif %}
        await dao.create_dummy_model(name=name)
        return name



from {{cookiecutter.project_name}}.settings import settings

from piccolo.conf.apps import AppRegistry


{%- if cookiecutter.db_info.name == "postgresql" %}
from piccolo.engine.postgres import PostgresEngine

DB = PostgresEngine(
    config={
        "database": settings.db_base,
        "user": settings.db_user,
        "password": settings.db_pass,
        "host": settings.db_host,
        "port": settings.db_port,
    }
)



{%- elif cookiecutter.db_info.name == "sqlite" %}
from piccolo.engine.sqlite import SQLiteEngine

DB = SQLiteEngine(path=str(settings.db_file))
{%- endif %}


APP_REGISTRY = AppRegistry(
    apps=["{{cookiecutter.project_name}}.db.app_conf"]
)



from fastapi import Depends
from strawberry.fastapi import BaseContext

{%- if cookiecutter.enable_redis == "True" %}
from redis.asyncio import ConnectionPool
from {{cookiecutter.project_name}}.services.redis.dependency import get_redis_pool
{%- endif %}

{%- if cookiecutter.enable_rmq == "True" %}
from aio_pika.pool import Pool
from aio_pika import Channel
from {{cookiecutter.project_name}}.services.rabbit.dependencies import get_rmq_channel_pool
{%- endif %}

{%- if cookiecutter.enable_kafka == "True" %}
from aiokafka import AIOKafkaProducer
from {{cookiecutter.project_name}}.services.kafka.dependencies import get_kafka_producer
{%- endif %}


{%- if cookiecutter.orm == "sqlalchemy" %}
from sqlalchemy.ext.asyncio import AsyncSession
from {{cookiecutter.project_name}}.db.dependencies import get_db_session
{%- elif cookiecutter.orm == "psycopg" %}
from psycopg_pool import AsyncConnectionPool
from {{cookiecutter.project_name}}.db.dependencies import get_db_pool
{%- endif %}


class Context(BaseContext):
    """Global graphql context."""

    def __init__(
        self,
        {%- if cookiecutter.enable_redis == "True" %}
        redis_pool: ConnectionPool = Depends(get_redis_pool),
        {%- endif %}
        {%- if cookiecutter.enable_rmq == "True" %}
        rabbit: Pool[Channel] = Depends(get_rmq_channel_pool),
        {%- endif %}
        {%- if cookiecutter.orm == "sqlalchemy" %}
        db_connection: AsyncSession = Depends(get_db_session),
        {%- elif cookiecutter.orm == "psycopg" %}
        db_pool: AsyncConnectionPool = Depends(get_db_pool),
        {%- endif %}
        {%- if cookiecutter.enable_kafka == "True" %}
        kafka_producer: AIOKafkaProducer = Depends(get_kafka_producer),
        {%- endif %}
    ) -> None:
        {%- if cookiecutter.enable_redis == "True" %}
        self.redis_pool = redis_pool
        {%- endif %}
        {%- if cookiecutter.enable_rmq == "True" %}
        self.rabbit = rabbit
        {%- endif %}
        {%- if cookiecutter.orm  == "sqlalchemy" %}
        self.db_connection = db_connection
        {%- endif %}
        {%- if cookiecutter.orm == "psycopg" %}
        self.db_pool = db_pool
        {%- endif %}
        {%- if cookiecutter.enable_kafka == "True" %}
        self.kafka_producer = kafka_producer
        {%- endif %}
        pass  # noqa: WPS420


def get_context(context: Context = Depends(Context)) -> Context:
    """
    Get custom context.

    :param context: graphql context.
    :return: context
    """
    return context



from typing import AsyncGenerator

from sqlalchemy.ext.asyncio import AsyncSession
from starlette.requests import Request


async def get_db_session(request: Request) -> AsyncGenerator[AsyncSession, None]:
    """
    Create and get database session.

    :param request: current request.
    :yield: database session.
    """
    session: AsyncSession = request.app.state.db_session_factory()

    try:  # noqa: WPS501
        yield session
    finally:
        await session.commit()
        await session.close()



from fastapi import APIRouter, Request
from fastapi.responses import HTMLResponse
from fastapi.openapi.docs import (
    get_redoc_html,
    get_swagger_ui_html,
    get_swagger_ui_oauth2_redirect_html,
)

router = APIRouter()

@router.get("/docs", include_in_schema=False)
async def swagger_ui_html(request: Request) -> HTMLResponse:
    """
    Swagger UI.

    :param request: current request.
    :return: rendered swagger UI.
    """
    title = request.app.title
    return get_swagger_ui_html(
        openapi_url=request.app.openapi_url,
        title=f"{title} - Swagger UI",
        oauth2_redirect_url=request.url_for("swagger_ui_redirect"),
        swagger_js_url="/static/docs/swagger-ui-bundle.js",
        swagger_css_url="/static/docs/swagger-ui.css",
    )


@router.get("/swagger-redirect", include_in_schema=False)
async def swagger_ui_redirect() -> HTMLResponse:
    """
    Redirect to swagger.

    :return: redirect.
    """
    return get_swagger_ui_oauth2_redirect_html()


@router.get("/redoc", include_in_schema=False)
async def redoc_html(request: Request) -> HTMLResponse:
    """
    Redoc UI.

    :param request: current request.
    :return: rendered redoc UI.
    """
    title = request.app.title
    return get_redoc_html(
        openapi_url=request.app.openapi_url,
        title=f"{title} - ReDoc",
        redoc_js_url="/static/docs/redoc.standalone.js",
    )



from fastapi.routing import APIRouter

{%- if cookiecutter.enable_routers == "True" %}
{%- if cookiecutter.api_type == 'rest' %}
from {{cookiecutter.project_name}}.web.api import echo
{%- if cookiecutter.add_dummy == 'True' %}
from {{cookiecutter.project_name}}.web.api import dummy
{%- endif %}
{%- if cookiecutter.enable_redis == "True" %}
from {{cookiecutter.project_name}}.web.api import redis
{%- endif %}
{%- if cookiecutter.enable_rmq == "True" %}
from {{cookiecutter.project_name}}.web.api import rabbit
{%- endif %}
{%- if cookiecutter.enable_kafka == "True" %}
from {{cookiecutter.project_name}}.web.api import kafka
{%- endif %}
{%- endif %}
{%- endif %}
{%- if cookiecutter.self_hosted_swagger == "True" %}
from {{cookiecutter.project_name}}.web.api import docs
{%- endif %}
from {{cookiecutter.project_name}}.web.api import monitoring

api_router = APIRouter()
api_router.include_router(monitoring.router)
{%- if cookiecutter.self_hosted_swagger == "True" %}
api_router.include_router(docs.router)
{%- endif %}
{%- if cookiecutter.enable_routers == "True" %}
{%- if cookiecutter.api_type == 'rest' %}
api_router.include_router(echo.router, prefix="/echo", tags=["echo"])
{%- if cookiecutter.add_dummy == 'True' %}
api_router.include_router(dummy.router, prefix="/dummy", tags=["dummy"])
{%- endif %}
{%- if cookiecutter.enable_redis == "True" %}
api_router.include_router(redis.router, prefix="/redis", tags=["redis"])
{%- endif %}
{%- if cookiecutter.enable_rmq == "True" %}
api_router.include_router(rabbit.router, prefix="/rabbit", tags=["rabbit"])
{%- endif %}
{%- if cookiecutter.enable_kafka == "True" %}
api_router.include_router(kafka.router, prefix="/kafka", tags=["kafka"])
{%- endif %}
{%- endif %}
{%- endif %}



from aiokafka import AIOKafkaProducer
from fastapi import APIRouter, Depends

from {{cookiecutter.project_name}}.services.kafka.dependencies import get_kafka_producer
from {{cookiecutter.project_name}}.web.api.kafka.schema import KafkaMessage

router = APIRouter()


@router.post("/")
async def send_kafka_message(
    kafka_message: KafkaMessage,
    producer: AIOKafkaProducer = Depends(get_kafka_producer),
) -> None:
    """
    Sends message to kafka.

    :param producer: kafka's producer.
    :param kafka_message: message to publish.
    """
    await producer.send(
        topic=kafka_message.topic,
        value=kafka_message.message.encode(),
    )



from fastapi import FastAPI
from fastapi.responses import UJSONResponse
import logging
from {{cookiecutter.project_name}}.web.api.router import api_router
from {{cookiecutter.project_name}}.settings import settings
{%- if cookiecutter.api_type == 'graphql' %}
from {{cookiecutter.project_name}}.web.gql.router import gql_router
{%- endif %}
from {{cookiecutter.project_name}}.web.lifetime import register_startup_event, register_shutdown_event
from importlib import metadata

{%- if cookiecutter.orm == 'tortoise' %}
from tortoise.contrib.fastapi import register_tortoise
from {{cookiecutter.project_name}}.db.config import TORTOISE_CONFIG
{%- endif %}

{%- if cookiecutter.sentry_enabled == "True" %}
import sentry_sdk
from sentry_sdk.integrations.logging import LoggingIntegration
from sentry_sdk.integrations.fastapi import FastApiIntegration
{%- if cookiecutter.orm == "sqlalchemy" %}
from sentry_sdk.integrations.sqlalchemy import SqlalchemyIntegration
{%- endif %}
{%- endif %}

{%- if cookiecutter.enable_loguru == "True" %}
from {{cookiecutter.project_name}}.logging import configure_logging
{%- endif %}

{%- if cookiecutter.self_hosted_swagger == 'True' %}
from fastapi.staticfiles import StaticFiles
from pathlib import Path


APP_ROOT = Path(__file__).parent.parent
{%- endif %}


def get_app() -> FastAPI:
    """
    Get FastAPI application.

    This is the main constructor of an application.

    :return: application.
    """
    {%- if cookiecutter.enable_loguru == "True" %}
    configure_logging()
    {%- endif %}
    {%- if cookiecutter.sentry_enabled == "True" %}
    if settings.sentry_dsn:
        # Enables sentry integration.
        sentry_sdk.init(
            dsn=settings.sentry_dsn,
            traces_sample_rate=settings.sentry_sample_rate,
            environment=settings.environment,
            integrations=[
                FastApiIntegration(transaction_style="endpoint"),
                LoggingIntegration(
                    level=logging.getLevelName(
                        settings.log_level.value,
                    ),
                    event_level=logging.ERROR,
                ),
                {%- if cookiecutter.orm == "sqlalchemy" %}
                SqlalchemyIntegration(),
                {%- endif %}
            ],
        )
    {%- endif %}
    app = FastAPI(
        title="{{cookiecutter.project_name}}",
        version=metadata.version("{{cookiecutter.project_name}}"),
        {%- if cookiecutter.self_hosted_swagger == 'True' %}
        docs_url=None,
        redoc_url=None,
        {% else %}
        docs_url="/api/docs",
        redoc_url="/api/redoc",
        {%- endif %}
        openapi_url="/api/openapi.json",
        default_response_class=UJSONResponse,
    )

    # Adds startup and shutdown events.
    register_startup_event(app)
    register_shutdown_event(app)

    # Main router for the API.
    app.include_router(router=api_router, prefix="/api")
    {%- if cookiecutter.api_type == 'graphql' %}
    # Graphql router
    app.include_router(router=gql_router, prefix="/graphql")
    {%- endif %}

    {%- if cookiecutter.self_hosted_swagger == 'True' %}
    # Adds static directory.
    # This directory is used to access swagger files.
    app.mount(
        "/static",
        StaticFiles(directory=APP_ROOT / "static"),
        name="static"
    )
    {% endif %}

    {%- if cookiecutter.orm == 'tortoise' %}
    # Configures tortoise orm.
    register_tortoise(
        app,
        config=TORTOISE_CONFIG,
        add_exception_handlers=True,
        {%- if cookiecutter.enable_migrations != "True" %}
        generate_schemas=True,
        {%- endif %}
    )
    {%- endif %}

    return app



from fastapi import FastAPI
from httpx import AsyncClient
from starlette import status
import pytest

@pytest.mark.anyio
async def test_health(client: AsyncClient, fastapi_app: FastAPI) -> None:
    """
    Checks the health endpoint.

    :param client: client for the app.
    :param fastapi_app: current FastAPI application.
    """
    url = fastapi_app.url_path_for('health_check')
    response = await client.get(url)
    assert response.status_code == status.HTTP_200_OK



from fastapi import APIRouter

router = APIRouter()

@router.get('/health')
def health_check() -> None:
    """
    Checks the health of a project.

    It returns 200 if the project is healthy.
    """



import uuid

import pytest
from aio_pika import Channel
from aio_pika.abc import AbstractQueue
from aio_pika.exceptions import QueueEmpty
from aio_pika.pool import Pool
from fastapi import FastAPI
from httpx import AsyncClient


@pytest.mark.anyio
async def test_message_publishing(
    fastapi_app: FastAPI,
    client: AsyncClient,
    test_queue: AbstractQueue,
    test_exchange_name: str,
    test_routing_key: str,
) -> None:
    """
    Tests that message is published correctly.

    It sends message to rabbitmq and reads it
    from binded queue.
    """
    message_text = uuid.uuid4().hex
    {%- if cookiecutter.api_type == 'rest' %}
    url = fastapi_app.url_path_for("send_rabbit_message")
    await client.post(
        url,
        json={
            "exchange_name": test_exchange_name,
            "routing_key": test_routing_key,
            "message": message_text,
        },
    )
    {%- elif cookiecutter.api_type == 'graphql' %}
    url = fastapi_app.url_path_for('handle_http_post')
    await client.post(
        url,
        json={
            "query": "mutation($message:RabbitMessageDTO!)"
                     "{sendRabbitMessage(message:$message)}",
            "variables": {
                "message": {
                    "exchangeName": test_exchange_name,
                    "routingKey": test_routing_key,
                    "message": message_text,
                },
            },
        },
    )
    {%- endif %}
    message = await test_queue.get(timeout=1)
    assert message is not None
    await message.ack()
    assert message.body.decode("utf-8") == message_text


@pytest.mark.anyio
async def test_message_wrong_exchange(
    fastapi_app: FastAPI,
    client: AsyncClient,
    test_queue: AbstractQueue,
    test_exchange_name: str,
    test_routing_key: str,
    test_rmq_pool: Pool[Channel],
) -> None:
    """
    Tests that message can be published in undeclared exchange.

    It sends message to random queue,
    tries to get message from binded queue
    and checks that new exchange were created.
    """
    random_exchange = uuid.uuid4().hex
    assert random_exchange != test_exchange_name
    message_text = uuid.uuid4().hex
    {%- if cookiecutter.api_type == 'rest' %}
    url = fastapi_app.url_path_for("send_rabbit_message")
    await client.post(
        url,
        json={
            "exchange_name": random_exchange,
            "routing_key": test_routing_key,
            "message": message_text,
        },
    )
    {%- elif cookiecutter.api_type == 'graphql' %}
    url = fastapi_app.url_path_for('handle_http_post')
    await client.post(
        url,
        json={
            "query": "mutation($message:RabbitMessageDTO!)"
                     "{sendRabbitMessage(message:$message)}",
            "variables": {
                "message": {
                    "exchangeName": random_exchange,
                    "routingKey": test_routing_key,
                    "message": message_text,
                },
            },
        },
    )
    {%- endif %}
    with pytest.raises(QueueEmpty):
        await test_queue.get(timeout=1)

    async with test_rmq_pool.acquire() as conn:
        exchange = await conn.get_exchange(random_exchange, ensure=True)
        await exchange.delete(if_unused=False)



from typing import List, Optional

from {{cookiecutter.project_name}}.db.models.dummy_model import DummyModel


class DummyDAO:
    """Class for accessing dummy table."""

    async def create_dummy_model(self, name: str) -> None:
        """
        Add single dummy to session.

        :param name: name of a dummy.
        """
        await DummyModel.objects.create(name=name)

    async def get_all_dummies(self, limit: int, offset: int) -> List[DummyModel]:
        """
        Get all dummy models with limit/offset pagination.

        :param limit: limit of dummies.
        :param offset: offset of dummies.
        :return: stream of dummies.
        """
        return await DummyModel.objects.limit(limit).offset(offset).all()

    async def filter(
        self,
        name: Optional[str] = None,
    ) -> List[DummyModel]:
        """
        Get specific dummy model.

        :param name: name of dummy instance.
        :return: dummy models.
        """
        query = DummyModel.objects
        if name:
            query = query.filter(DummyModel.name == name)
        return await query.all()



import strawberry
from strawberry.types import Info

from {{cookiecutter.project_name}}.web.gql.context import Context
from {{cookiecutter.project_name}}.web.gql.redis.schema import RedisDTO
from redis.asyncio import Redis

@strawberry.type
class Query:
    """Query to interact with redis."""

    @strawberry.field(description="Get value from redis")
    async def get_redis_value(self, key: str, info: Info[Context, None]) -> RedisDTO:
        """
        Gets value from redis.

        :param key: key to search for.
        :param info: resolver context.
        :return: information from redis.
        """
        async with Redis(connection_pool=info.context.redis_pool) as redis:
            val = await redis.get(name=key)
        if isinstance(val, bytes):
            val = val.decode("utf-8")
        return RedisDTO(key=key, value=val)  # type: ignore



import asyncio
from logging.config import fileConfig

from alembic import context
from sqlalchemy.ext.asyncio.engine import create_async_engine
from sqlalchemy.future import Connection

from {{cookiecutter.project_name}}.db.meta import meta
from {{cookiecutter.project_name}}.db.models import load_all_models
from {{cookiecutter.project_name}}.settings import settings

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config


load_all_models()
# Interpret the config file for Python logging.
# This line sets up loggers basically.
if config.config_file_name is not None:
    fileConfig(config.config_file_name)

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata
target_metadata = meta

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.


async def run_migrations_offline() -> None:
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    context.configure(
        url=str(settings.db_url),
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()


def do_run_migrations(connection: Connection) -> None:
    """
    Run actual sync migrations.

    :param connection: connection to the database.
    """
    context.configure(connection=connection, target_metadata=target_metadata)

    with context.begin_transaction():
        context.run_migrations()


async def run_migrations_online() -> None:
    """
    Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.
    """
    connectable = create_async_engine(str(settings.db_url))

    async with connectable.connect() as connection:
        await connection.run_sync(do_run_migrations)

loop = asyncio.get_event_loop()
if context.is_offline_mode():
    task = run_migrations_offline()
else:
    task = run_migrations_online()

loop.run_until_complete(task)



from typing import List

import strawberry
from strawberry.types import Info

from {{cookiecutter.project_name}}.db.dao.dummy_dao import DummyDAO
from {{cookiecutter.project_name}}.web.gql.context import Context
from {{cookiecutter.project_name}}.web.gql.dummy.schema import DummyModelDTO


@strawberry.type
class Query:
    """Query to interact with dummies."""

    @strawberry.field(description="Get all dummies")
    async def get_dummy_models(
        self,
        {%- if cookiecutter.orm in ["sqlalchemy", "psycopg"] %}
        info: Info[Context, None],
        {%- endif %}
        limit: int = 15,
        offset: int = 0,
    ) -> List[DummyModelDTO]:
        """
        Retrieves all dummy objects from database.

        {% if cookiecutter.orm in ["sqlalchemy", "psycopg"] -%}
        :param info: connection info.
        {% endif -%}
        :param limit: limit of dummy objects, defaults to 10.
        :param offset: offset of dummy objects, defaults to 0.
        :return: list of dummy obbjects from database.
        """
        {%- if cookiecutter.orm == "sqlalchemy" %}
        dao = DummyDAO(info.context.db_connection)
        {%- elif cookiecutter.orm == "psycopg" %}
        dao = DummyDAO(info.context.db_pool)
        {%- else %}
        dao = DummyDAO()
        {%- endif %}
        return await dao.get_all_dummies(limit=limit, offset=offset)  # type: ignore



from typing import List
from {{cookiecutter.project_name}}.settings import settings

MODELS_MODULES: List[str] = [{%- if cookiecutter.add_dummy == 'True' %}"{{cookiecutter.project_name}}.db.models.dummy_model"{%- endif %}]  # noqa: WPS407

TORTOISE_CONFIG = {  # noqa: WPS407
    "connections": {
        "default": str(settings.db_url),
    },
    "apps": {
        "models": {
            "models": MODELS_MODULES {%- if cookiecutter.enable_migrations == "True" %} + ["aerich.models"] {%- endif %} ,
            "default_connection": "default",
        },
    },
}



from aiokafka import AIOKafkaProducer
from fastapi import Request


def get_kafka_producer(request: Request) -> AIOKafkaProducer:  # pragma: no cover
    """
    Returns kafka producer.

    :param request: current request.
    :return: kafka producer from the state.
    """
    return request.app.state.kafka_producer



import re
import pytest
import os
import tempfile
import shutil
from faker import Faker
from pathlib import Path
from fastapi_template.input_model import BuilderContext, Database
from fastapi_template.tests.utils import run_docker_compose_command


@pytest.fixture
def project_name(worker_id: str) -> str:
    """
    Generate name for test project.

    :return: project name.
    """
    fake = Faker()
    raw_name = fake.name_female() + worker_id
    clear_name: str = (
        raw_name.lower().replace(" ", "_").replace("-", "_").replace(".", "_")
    )
    return re.sub("_+", "_", clear_name).strip("_")


@pytest.fixture(scope="session", autouse=True)
def generator_start_dir() -> str:
    """
    Generate directory to work into

    :yield: this fixture generates dir for all test projects.
    """
    old_cwd = os.getcwd()
    newpath = tempfile.mkdtemp()
    os.chdir(newpath)
    try:
        yield newpath
    finally:
        os.chdir(old_cwd)
        shutil.rmtree(newpath, ignore_errors=True)


@pytest.fixture()
def default_context(project_name: str) -> None:
    """
    Default builder context without features.

    :param project_name: current project name.
    :return: context.
    """
    return BuilderContext(
        project_name=project_name,
        kube_name=project_name.replace("_", "-"),
        api_type="rest",
        ci_type="none",
        db="none",
        db_info=Database(name="none").dict(),
        enable_redis=False,
        enable_migrations=False,
        enable_kube=False,
        enable_routers=True,
        add_dummy=False,
        self_hosted_swagger=False,
        enable_rmq=False,
        prometheus_enabled=False,
        otlp_enabled=False,
        sentry_enabled=False,
        force=True,
    )


@pytest.fixture(autouse=True)
def default_dir(generator_start_dir: str) -> None:
    """
    Change directory to generator_start_dir.

    :param generator_start_dir: start_dir.
    """
    yield
    cwd = os.getcwd()
    if cwd != generator_start_dir:
        os.chdir(generator_start_dir)


@pytest.fixture(autouse=True)
def docker_module_shutdown(generator_start_dir: str, project_name: str) -> None:
    """
    Cleans up docker context.

    :param generator_start_dir: generator dir.
    :param project_name: name of the project.
    """
    yield
    cwd = os.getcwd()
    project_dir = Path(generator_start_dir) / project_name
    if not project_dir.exists():
        return
    os.chdir(project_dir)
    Path("poetry.lock").unlink(missing_ok=True)
    run_docker_compose_command("down -v")
    os.chdir(cwd)



import os
from sqlalchemy import text
from sqlalchemy.engine import URL, make_url
from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession, AsyncEngine
from sqlalchemy.orm import sessionmaker
from {{cookiecutter.project_name}}.settings import settings

{% if cookiecutter.db_info.name == "postgresql" -%}
async def create_database() -> None:
    """Create a databse."""
    db_url = make_url(str(settings.db_url.with_path('/postgres')))
    engine = create_async_engine(db_url, isolation_level="AUTOCOMMIT")

    async with engine.connect() as conn:
        database_existance = await conn.execute(
            text(
                f"SELECT 1 FROM pg_database WHERE datname='{settings.db_base}'",  # noqa: E501, S608
            )
        )
        database_exists = database_existance.scalar() == 1

    if database_exists:
        await drop_database()

    async with engine.connect() as conn:  # noqa: WPS440
        await conn.execute(
            text(
                f'CREATE DATABASE "{settings.db_base}" ENCODING "utf8" TEMPLATE template1',  # noqa: E501
            )
        )

async def drop_database() -> None:
    """Drop current database."""
    db_url = make_url(str(settings.db_url.with_path('/postgres')))
    engine = create_async_engine(db_url, isolation_level="AUTOCOMMIT")
    async with engine.connect() as conn:
        disc_users = (
        "SELECT pg_terminate_backend(pg_stat_activity.pid) "  # noqa: S608
        "FROM pg_stat_activity "
        f"WHERE pg_stat_activity.datname = '{settings.db_base}' "
        "AND pid <> pg_backend_pid();"
        )
        await conn.execute(text(disc_users))
        await conn.execute(text(f'DROP DATABASE "{settings.db_base}"'))


{%- endif %}
{%- if cookiecutter.db_info.name == "mysql" %}
async def create_database() -> None:
    """Create a databse."""
    engine = create_async_engine(str(settings.db_url.with_path("/mysql")))

    async with engine.connect() as conn:
        database_existance = await conn.execute(
            text(
                "SELECT 1 FROM INFORMATION_SCHEMA.SCHEMATA"  # noqa: S608
                f" WHERE SCHEMA_NAME='{settings.db_base}';",
            )
        )
        database_exists = database_existance.scalar() == 1

    if database_exists:
        await drop_database()

    async with engine.connect() as conn:  # noqa: WPS440
        await conn.execute(
            text(
                f'CREATE DATABASE {settings.db_base};'
            )
        )

async def drop_database() -> None:
    """Drop current database."""
    engine = create_async_engine(str(settings.db_url.with_path("/mysql")))
    async with engine.connect() as conn:
        await conn.execute(text(f'DROP DATABASE {settings.db_base};'))
{%- endif %}
{%- if cookiecutter.db_info.name == "sqlite" %}
async def create_database() -> None:
    """Create a databse."""

async def drop_database() -> None:
    """Drop current database."""
    if settings.db_file.exists():
        os.remove(settings.db_file)

{%- endif %}



from typing import Awaitable, Callable
import logging
from fastapi import FastAPI

from {{cookiecutter.project_name}}.settings import settings

{%- if cookiecutter.prometheus_enabled == "True" %}
from prometheus_fastapi_instrumentator.instrumentation import PrometheusFastApiInstrumentator
{%- endif %}

{%- if cookiecutter.enable_redis == "True" %}
from {{cookiecutter.project_name}}.services.redis.lifetime import init_redis, shutdown_redis
{%- endif %}

{%- if cookiecutter.enable_rmq == "True" %}
from {{cookiecutter.project_name}}.services.rabbit.lifetime import init_rabbit, shutdown_rabbit
{%- endif %}

{%- if cookiecutter.enable_kafka == "True" %}
from {{cookiecutter.project_name}}.services.kafka.lifetime import init_kafka, shutdown_kafka
{%- endif %}


{%- if cookiecutter.orm == "ormar" %}
from {{cookiecutter.project_name}}.db.config import database
{%- if cookiecutter.db_info.name != "none" and cookiecutter.enable_migrations != "True" %}
from sqlalchemy.engine import create_engine
from {{cookiecutter.project_name}}.db.meta import meta
from {{cookiecutter.project_name}}.db.models import load_all_models
{%- endif %}
{%- endif %}

{%- if cookiecutter.otlp_enabled == "True" %}
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import (
    OTLPSpanExporter,
)
from opentelemetry.instrumentation.fastapi import FastAPIInstrumentor
from opentelemetry.sdk.resources import (
    SERVICE_NAME,
    TELEMETRY_SDK_LANGUAGE,
    DEPLOYMENT_ENVIRONMENT,
    Resource,
)
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor
from opentelemetry.trace import set_tracer_provider
{%- if cookiecutter.enable_redis == "True" %}
from opentelemetry.instrumentation.redis import RedisInstrumentor
{%- endif %}
{%- if cookiecutter.db_info.name == "postgresql" and cookiecutter.orm in ["ormar", "tortoise"] %}
from opentelemetry.instrumentation.asyncpg import AsyncPGInstrumentor
{%- endif %}
{%- if cookiecutter.orm == "sqlalchemy" %}
from opentelemetry.instrumentation.sqlalchemy import (
    SQLAlchemyInstrumentor,
)
{%- endif %}
{%- if cookiecutter.enable_rmq == "True" %}
from opentelemetry.instrumentation.aio_pika import AioPikaInstrumentor
{%- endif %}
{%- if cookiecutter.enable_loguru != "True" %}
from opentelemetry.instrumentation.logging import LoggingInstrumentor
{%- endif %}
{%- endif %}

{%- if cookiecutter.orm == "psycopg" %}
import psycopg_pool

async def _setup_db(app: FastAPI) -> None:
    """
    Creates connection pool for timescaledb.

    :param app: current FastAPI app.
    """
    app.state.db_pool = psycopg_pool.AsyncConnectionPool(conninfo=str(settings.db_url))
    await app.state.db_pool.wait()
{%- endif %}

{%- if cookiecutter.orm == "sqlalchemy" %}
from asyncio import current_task
from sqlalchemy.ext.asyncio import (
    AsyncSession,
    async_scoped_session,
    create_async_engine,
)
from sqlalchemy.orm import sessionmaker

{%- if cookiecutter.enable_migrations != "True" %}
from {{cookiecutter.project_name}}.db.meta import meta
from {{cookiecutter.project_name}}.db.models import load_all_models
{%- endif %}


def _setup_db(app: FastAPI) -> None:  # pragma: no cover
    """
    Creates connection to the database.

    This function creates SQLAlchemy engine instance,
    session_factory for creating sessions
    and stores them in the application's state property.

    :param app: fastAPI application.
    """
    engine = create_async_engine(str(settings.db_url), echo=settings.db_echo)
    session_factory = async_scoped_session(
        sessionmaker(
            engine,
            expire_on_commit=False,
            class_=AsyncSession,
        ),
        scopefunc=current_task,
    )
    app.state.db_engine = engine
    app.state.db_session_factory = session_factory
{%- endif %}

{%- if cookiecutter.enable_migrations != "True" %}
{%- if cookiecutter.orm in ["ormar", "sqlalchemy"] %}
async def _create_tables() -> None:  # pragma: no cover
    """Populates tables in the database."""
    load_all_models()
    {%- if cookiecutter.orm == "ormar" %}
    engine = create_engine(str(settings.db_url))
    with engine.connect() as connection:
        meta.create_all(connection)
    engine.dispose()
    {%- elif cookiecutter.orm == "sqlalchemy" %}
    engine = create_async_engine(str(settings.db_url))
    async with engine.begin() as connection:
        await connection.run_sync(meta.create_all)
    await engine.dispose()
    {%- endif %}
{%- endif %}
{%- endif %}

{%- if cookiecutter.otlp_enabled == "True" %}
def setup_opentelemetry(app: FastAPI) -> None:  # pragma: no cover
    """
    Enables opentelemetry instrumentation.

    :param app: current application.
    """
    if not settings.opentelemetry_endpoint:
        return

    tracer_provider = TracerProvider(
        resource=Resource(
            attributes={
                SERVICE_NAME: "{{cookiecutter.project_name}}",
                TELEMETRY_SDK_LANGUAGE: "python",
                DEPLOYMENT_ENVIRONMENT: settings.environment,
            }
        )
    )

    tracer_provider.add_span_processor(
        BatchSpanProcessor(
            OTLPSpanExporter(
                endpoint=settings.opentelemetry_endpoint,
                insecure=True,
            )
        )
    )

    excluded_endpoints = [
        app.url_path_for('health_check'),
        app.url_path_for('openapi'),
        app.url_path_for('swagger_ui_html'),
        app.url_path_for('swagger_ui_redirect'),
        app.url_path_for('redoc_html'),
        {%- if cookiecutter.prometheus_enabled == "True" %}
        "/metrics",
        {%- endif %}
    ]

    FastAPIInstrumentor().instrument_app(
        app,
        tracer_provider=tracer_provider,
        excluded_urls=",".join(excluded_endpoints),
    )
    {%- if cookiecutter.enable_redis == "True" %}
    RedisInstrumentor().instrument(
        tracer_provider=tracer_provider,
    )
    {%- endif %}
    {%- if cookiecutter.db_info.name == "postgresql" and cookiecutter.orm in ["ormar", "tortoise"] %}
    AsyncPGInstrumentor().instrument(
        tracer_provider=tracer_provider,
    )
    {%- endif %}
    {%- if cookiecutter.orm == "sqlalchemy" %}
    SQLAlchemyInstrumentor().instrument(
        tracer_provider=tracer_provider,
        engine=app.state.db_engine.sync_engine,
    )
    {%- endif %}
    {%- if cookiecutter.enable_rmq == "True" %}
    AioPikaInstrumentor().instrument(
        tracer_provider=tracer_provider,
    )
    {%- endif %}
    {%- if cookiecutter.enable_loguru != "True" %}
    LoggingInstrumentor().instrument(
        tracer_provider=tracer_provider,
        set_logging_format=True,
        log_level=logging.getLevelName(settings.log_level.value),
    )
    {%- endif %}

    set_tracer_provider(tracer_provider=tracer_provider)


def stop_opentelemetry(app: FastAPI) -> None:  # pragma: no cover
    """
    Disables opentelemetry instrumentation.

    :param app: current application.
    """
    if not settings.opentelemetry_endpoint:
        return

    FastAPIInstrumentor().uninstrument_app(app)
    {%- if cookiecutter.enable_redis == "True" %}
    RedisInstrumentor().uninstrument()
    {%- endif %}
    {%- if cookiecutter.db_info.name == "postgresql" and cookiecutter.orm in ["ormar", "tortoise"] %}
    AsyncPGInstrumentor().uninstrument()
    {%- endif %}
    {%- if cookiecutter.orm == "sqlalchemy" %}
    SQLAlchemyInstrumentor().uninstrument()
    {%- endif %}
    {%- if cookiecutter.enable_rmq == "True" %}
    AioPikaInstrumentor().uninstrument()
    {%- endif %}

{%- endif %}

{%- if cookiecutter.prometheus_enabled == "True" %}
def setup_prometheus(app: FastAPI) -> None:  # pragma: no cover
    """
    Enables prometheus integration.

    :param app: current application.
    """
    PrometheusFastApiInstrumentator(should_group_status_codes=False).instrument(
        app,
    ).expose(app, should_gzip=True, name="prometheus_metrics")
{%- endif %}


def register_startup_event(app: FastAPI) -> Callable[[], Awaitable[None]]:  # pragma: no cover
    """
    Actions to run on application startup.

    This function uses fastAPI app to store data
    inthe state, such as db_engine.

    :param app: the fastAPI application.
    :return: function that actually performs actions.
    """

    @app.on_event("startup")
    async def _startup() -> None:  # noqa: WPS430
        {%- if cookiecutter.orm == "sqlalchemy" %}
        _setup_db(app)
        {%- elif cookiecutter.orm == "ormar" %}
        await database.connect()
        {%- elif cookiecutter.orm == "psycopg" %}
        await _setup_db(app)
        {%- endif %}
        {%- if cookiecutter.db_info.name != "none" and cookiecutter.enable_migrations != "True" %}
        {%- if cookiecutter.orm in ["ormar", "sqlalchemy"] %}
        await _create_tables()
        {%- endif %}
        {%- endif %}
        {%- if cookiecutter.otlp_enabled == "True" %}
        setup_opentelemetry(app)
        {%- endif %}
        {%- if cookiecutter.enable_redis == "True" %}
        init_redis(app)
        {%- endif %}
        {%- if cookiecutter.enable_rmq == "True" %}
        init_rabbit(app)
        {%- endif %}
        {%- if cookiecutter.enable_kafka == "True" %}
        await init_kafka(app)
        {%- endif %}
        {%- if cookiecutter.prometheus_enabled == "True" %}
        setup_prometheus(app)
        {%- endif %}
        pass  # noqa: WPS420

    return _startup


def register_shutdown_event(app: FastAPI) -> Callable[[], Awaitable[None]]:  # pragma: no cover
    """
    Actions to run on application's shutdown.

    :param app: fastAPI application.
    :return: function that actually performs actions.
    """

    @app.on_event("shutdown")
    async def _shutdown() -> None:  # noqa: WPS430
        {%- if cookiecutter.orm == "sqlalchemy" %}
        await app.state.db_engine.dispose()
        {% elif cookiecutter.orm == "ormar" %}
        await database.disconnect()
        {%- elif cookiecutter.orm == "psycopg" %}
        await app.state.db_pool.close()
        {%- endif %}
        {%- if cookiecutter.enable_redis == "True" %}
        await shutdown_redis(app)
        {%- endif %}
        {%- if cookiecutter.enable_rmq == "True" %}
        await shutdown_rabbit(app)
        {%- endif %}
        {%- if cookiecutter.enable_kafka == "True" %}
        await shutdown_kafka(app)
        {%- endif %}
        {%- if cookiecutter.otlp_enabled == "True" %}
        stop_opentelemetry(app)
        {%- endif %}
        pass  # noqa: WPS420

    return _shutdown



import uuid
import fakeredis
import pytest

from fastapi import FastAPI
from httpx import AsyncClient
from starlette import status
from redis.asyncio import ConnectionPool, Redis


@pytest.mark.anyio
async def test_setting_value(
    fastapi_app: FastAPI,
    fake_redis_pool: ConnectionPool,
    client: AsyncClient,
) -> None:
    """
    Tests that you can set value in redis.

    :param fastapi_app: current application fixture.
    :param fake_redis_pool: fake redis pool.
    :param client: client fixture.
    """
    {%- if cookiecutter.api_type == 'rest' %}
    url = fastapi_app.url_path_for('set_redis_value')
    {%- elif cookiecutter.api_type == 'graphql' %}
    url = fastapi_app.url_path_for('handle_http_post')
    {%- endif %}

    test_key = uuid.uuid4().hex
    test_val = uuid.uuid4().hex
    {%- if cookiecutter.api_type == 'rest' %}
    response = await client.put(url, json={
        "key": test_key,
        "value": test_val
    })
    {%- elif cookiecutter.api_type == 'graphql' %}
    query = """
    mutation ($key: String!, $val: String!) {
        setRedisValue(data: { key: $key, value: $val }) {
            key
            value
        }
    }
    """
    response = await client.post(
        url,
        json={
            "query": query,
            "variables": {"key": test_key, "val": test_val},
        },
    )
    {%- endif %}

    assert response.status_code == status.HTTP_200_OK
    async with Redis(connection_pool=fake_redis_pool) as redis:
        actual_value = await redis.get(test_key)
    assert actual_value.decode() == test_val


@pytest.mark.anyio
async def test_getting_value(
    fastapi_app: FastAPI,
    fake_redis_pool: ConnectionPool,
    client: AsyncClient,
) -> None:
    """
    Tests that you can get value from redis by key.

    :param fastapi_app: current application fixture.
    :param fake_redis_pool: fake redis pool.
    :param client: client fixture.
    """
    test_key = uuid.uuid4().hex
    test_val = uuid.uuid4().hex
    async with Redis(connection_pool=fake_redis_pool) as redis:
        await redis.set(test_key, test_val)

    {%- if cookiecutter.api_type == 'rest' %}
    url = fastapi_app.url_path_for('get_redis_value')
    {%- elif cookiecutter.api_type == 'graphql' %}
    url = fastapi_app.url_path_for('handle_http_post')
    {%- endif %}

    {%- if cookiecutter.api_type == 'rest' %}
    response = await client.get(url, params={"key": test_key})

    assert response.status_code == status.HTTP_200_OK
    assert response.json()['key'] == test_key
    assert response.json()['value'] == test_val
    {%- elif cookiecutter.api_type == 'graphql' %}
    response = await client.post(
        url,
        json={
            "query": "query($key:String!){redis:getRedisValue(key:$key){key value}}",
            "variables": {
                "key": test_key,
            },
        },
    )

    assert response.status_code == status.HTTP_200_OK
    assert response.json()["data"]["redis"]["key"] == test_key
    assert response.json()["data"]["redis"]["value"] == test_val
    {%- endif %}




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fastapi_template',
 'fastapi_template.template.hooks',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_ormar',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_ormar.dao',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_ormar.migrations',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_ormar.migrations.versions',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_ormar.models',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_piccolo',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_piccolo.dao',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_piccolo.migrations',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_piccolo.models',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_psycopg',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_psycopg.dao',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_psycopg.models',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_sa',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_sa.dao',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_sa.migrations',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_sa.migrations.versions',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_sa.models',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_tortoise',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_tortoise.dao',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_tortoise.models',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.services',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.services.kafka',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.services.rabbit',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.services.redis',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.tests',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.api',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.api.docs',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.api.dummy',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.api.echo',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.api.kafka',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.api.monitoring',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.api.rabbit',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.api.redis',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.gql',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.gql.dummy',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.gql.echo',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.gql.kafka',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.gql.rabbit',
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.web.gql.redis',
 'fastapi_template.tests']

package_data = \
{'': ['*'],
 'fastapi_template': ['template/*',
                      'template/{{cookiecutter.project_name}}/*',
                      'template/{{cookiecutter.project_name}}/.github/workflows/*',
                      'template/{{cookiecutter.project_name}}/deploy/*',
                      'template/{{cookiecutter.project_name}}/deploy/kube/*'],
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}': ['static/docs/*'],
 'fastapi_template.template.{{cookiecutter.project_name}}.{{cookiecutter.project_name}}.db_tortoise': ['migrations/models/*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'cookiecutter>=1.7.3,<2.0.0',
 'pre-commit>=2.14.0,<3.0.0',
 'prompt-toolkit>=3.0.36,<4.0.0',
 'pydantic>=1.8.2,<2.0.0',
 'simple-term-menu>=1.5.2,<2.0.0',
 'termcolor>=1.1.0,<2.0.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['fastapi_template = fastapi_template.__main__:main']}

setup_kwargs = {
    'name': 'fastapi-template',
    'version': '4.0.0',
    'description': 'Feature-rich robust FastAPI template',
    'long_description': '![python version](https://img.shields.io/pypi/pyversions/fastapi_template?style=for-the-badge) ![Build status](https://img.shields.io/github/workflow/status/s3rius/FastAPI-template/Release%20python%20package?style=for-the-badge) [![version](https://img.shields.io/pypi/v/fastapi_template?style=for-the-badge)](https://pypi.org/project/fastapi-template/)\n[![](https://img.shields.io/pypi/dm/fastapi_template?style=for-the-badge)](https://pypi.org/project/fastapi-template/)\n<div align="center">\n<img src="https://raw.githubusercontent.com/s3rius/FastAPI-template/master/images/logo.png" width=700>\n<div><i>Flexible general-purpose template for FastAPI.</i></div>\n</div>\n\n## Usage\n\n⚠️ [Git](https://git-scm.com/downloads), [Python](https://www.python.org/) and [Poetry](https://python-poetry.org/) must be installed and accessible ⚠️\n\nPoetry version must be greater or equal than 1.1.8. Otherwise it won\'t be able to install SQLAlchemy.\n\n<div align="center">\n <a href="https://asciinema.org/a/ig0oi0fOq1hxqnW5X49XaaHIT" target="_blank"><img src="https://asciinema.org/a/ig0oi0fOq1hxqnW5X49XaaHIT.svg" /></a>\n  <p>Templator in action</p>\n</div>\n\nYou can install it directly from pypi with pip.\n```bash\npython3 -m pip install fastapi_template\npython3 -m fastapi_template\n# or fastapi_template\n# Answer all the questions\n# 🍪 Enjoy your new project 🍪\ncd new_project\ndocker-compose -f deploy/docker-compose.yml --project-directory . build\ndocker-compose -f deploy/docker-compose.yml --project-directory . up --build\n```\n\nIf you want to install it from sources, try this:\n```shell\npython3 -m pip install poetry\npython3 -m pip install .\npython3 -m fastapi_template\n```\n\nAlso you can use it with docker.\n```bash\ndocker run --rm -it -v "$(pwd):/projects" s3rius/fastapi_template\n```\n\n## Features\n\nOne of the coolest features is that this project is extremely configurable.\nYou can choose between different databases and even ORMs, or\nyou can even generate a project without a database!\nCurrently SQLAlchemy1.4, TortoiseORM, Piccolo and Ormar are supported.\n\nThis project can run as TUI or CLI and has excellent code documentation.\n\nGenerator features:\n- You can choose between GraphQL and REST api;\n- Different databases support;\n- Different ORMs support;\n- Optional migrations for each ORM except raw drivers;\n- Optional redis support;\n- Optional rabbitmq support;\n- different CI\\CD;\n- Optional Kubernetes config generation;\n- Optional Demo routers and models (This helps you to see how project is structured);\n- Pre-commit integration;\n- Generated tests with almost 90% coverage;\n- Tests for the generator itself;\n- Optional Prometheus integration;\n- Optional Sentry integration;\n- Optional Loguru logger;\n- Optional Opentelemetry integration.\n\n\nThis project can handle arguments passed through command line.\n\n```shell\n$ python -m fastapi_template --help\n\nUsage: fastapi_template [OPTIONS]\n\nOptions:\n  -n, --name TEXT                 Name of your awesome project\n  -V, --version                   Prints current version\n  --force                         Owerrite directory if it exists\n  --quite                         Do not ask for features during generation\n  --api-type [rest|graphql]       Select API type for your application\n  --db [none|sqlite|mysql|postgresql]\n                                  Select a database for your app\n  --orm [none|ormar|sqlalchemy|tortoise|psycopg|piccolo]\n                                  Choose Object–Relational Mapper lib\n  --ci [none|gitlab_ci|github]    Select a CI for your app\n  --redis                         Add redis support\n  --rabbit                        Add RabbitMQ support\n  --migrations                    Add Migrations\n  --kube                          Add kubernetes configs\n  --dummy                         Add dummy model\n  --routers                       Add example routers\n  --swagger                       Add self hosted swagger\n  --prometheus                    Add prometheus compatible metrics\n  --sentry                        Add sentry integration\n  --loguru                        Add loguru logger\n  --opentelemetry                 Add opentelemetry integration\n  --traefik                       Adds traefik labels to docker container\n  --kafka                         Add Kafka support\n  --help                          Show this message and exit.\n```\n',
    'author': 'Pavel Kirilin',
    'author_email': 'win10@list.ru',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/s3rius/FastAPI-template',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Optional
from fastapi_template.tests.utils import run_default_check
import pytest

from fastapi_template.input_model import BuilderContext
from fastapi_template.cli import db_menu


def init_context(
    context: BuilderContext,
    db: str,
    orm: Optional[str],
    api: Optional[str] = None,
) -> BuilderContext:
    db_info = None
    for entry in db_menu.entries:
        if entry.code == db:
            db_info = entry.additional_info.dict()
    if db_info is None:
        raise ValueError(f"Unknown database: {db}")

    context.db = db
    context.db_info = db_info
    context.orm = orm

    if api is not None:
        context.api_type = api

    context.enable_migrations = db != "none"
    context.add_dummy = db != "none"

    return context


def test_default_without_db(default_context: BuilderContext):
    run_default_check(init_context(default_context, "none", None))


@pytest.mark.parametrize(
    "db",
    [
        "postgresql",
        "sqlite",
        "mysql",
    ],
)
@pytest.mark.parametrize(
    "orm",
    [
        "sqlalchemy",
        "tortoise",
        "ormar",
        "piccolo",
    ],
)
def test_default_with_db(default_context: BuilderContext, db: str, orm: str):
    if orm == "piccolo" and db == "mysql":
        return
    run_default_check(init_context(default_context, db, orm))


@pytest.mark.parametrize("api", ["rest", "graphql"])
@pytest.mark.parametrize(
    "orm",
    [
        "sqlalchemy",
        "tortoise",
        "ormar",
        "piccolo",
    ],
)
def test_default_for_apis(default_context: BuilderContext, orm: str, api: str):
    run_default_check(init_context(default_context, "postgresql", orm, api))


@pytest.mark.parametrize(
    "orm",
    [
        "psycopg",
    ],
)
def test_pg_drivers(default_context: BuilderContext, orm: str):
    run_default_check(init_context(default_context, "postgresql", orm))


@pytest.mark.parametrize(
    "orm",
    [
        "sqlalchemy",
        "tortoise",
        "ormar",
        "psycopg",
        "piccolo",
    ],
)
def test_without_routers(default_context: BuilderContext, orm: str):
    context = init_context(default_context, "postgresql", orm)
    context.enable_routers = False
    run_default_check(context)


@pytest.mark.parametrize(
    "orm",
    [
        "sqlalchemy",
        "tortoise",
        "ormar",
        "piccolo",
    ],
)
def test_without_migrations(default_context: BuilderContext, orm: str):
    context = init_context(default_context, "postgresql", orm)
    context.enable_migrations = False
    run_default_check(context)


def test_with_selfhosted_swagger(default_context: BuilderContext):
    default_context.self_hosted_swagger = True
    run_default_check(default_context)


@pytest.mark.parametrize(
    "orm",
    [
        "sqlalchemy",
        "tortoise",
        "ormar",
        "psycopg",
        "piccolo",
    ],
)
def test_without_dummy(default_context: BuilderContext, orm: str):
    context = init_context(default_context, "postgresql", orm)
    context.add_dummy = False
    run_default_check(context)


@pytest.mark.parametrize(
    "api",
    [
        "rest",
        "graphql",
    ],
)
def test_redis(default_context: BuilderContext, api: str):
    default_context.enable_redis = True
    default_context.api_type = api
    run_default_check(default_context)


@pytest.mark.parametrize(
    "api",
    [
        "rest",
        "graphql",
    ],
)
def test_rmq(default_context: BuilderContext, api: str):
    default_context.enable_rmq = True
    default_context.api_type = api
    run_default_check(default_context)


def test_telemetry_pre_commit(default_context: BuilderContext):
    default_context.enable_rmq = True
    default_context.enable_redis = True
    default_context.prometheus_enabled = True
    default_context.otlp_enabled = True
    default_context.sentry_enabled = True
    default_context.enable_loguru = True
    run_default_check(default_context, without_pytest=True)


# @pytest.mark.parametrize("api", ["rest", "graphql"])
# def test_kafka(default_context: BuilderContext, api: str):
#     default_context.enable_kafka = True
#     default_context.api_type = api
#     run_default_check(default_context)



from aiokafka import AIOKafkaProducer
from fastapi import FastAPI

from {{cookiecutter.project_name}}.settings import settings


async def init_kafka(app: FastAPI) -> None:  # pragma: no cover
    """
    Initialize kafka producer.

    This function creates producer
    and makes initial connection to
    the kafka cluster. After that you
    can use producer stored in state.

    We don't need to use pools here,
    because aiokafka has implicit pool
    inside the producer.

    :param app: current application.
    """
    app.state.kafka_producer = AIOKafkaProducer(
        bootstrap_servers=settings.kafka_bootstrap_servers,
    )
    await app.state.kafka_producer.start()


async def shutdown_kafka(app: FastAPI) -> None:  # pragma: no cover
    """
    Shutdown kafka client.

    This function closes all connections
    and sends all pending data to kafka.

    :param app: current application.
    """
    await app.state.kafka_producer.stop()



from piccolo.apps.migrations.auto.migration_manager import MigrationManager
from piccolo.columns.column_types import Varchar
from piccolo.columns.indexes import IndexMethod


ID = "2022-04-16T17:38:51:672827"
VERSION = "0.74.0"
DESCRIPTION = "Creates dummy model"


async def forwards() -> MigrationManager:
    manager = MigrationManager(
        migration_id=ID, app_name="ptest_db", description=DESCRIPTION
    )

    manager.add_table("DummyModel", tablename="dummy_model")

    manager.add_column(
        table_class_name="DummyModel",
        tablename="dummy_model",
        column_name="name",
        db_column_name="name",
        column_class_name="Varchar",
        column_class=Varchar,
        params={
            "length": 200,
            "default": "",
            "null": False,
            "primary_key": False,
            "unique": False,
            "index": False,
            "index_method": IndexMethod.btree,
            "choices": None,
            "db_column_name": None,
            "secret": False,
        },
    )

    return manager



import strawberry


@strawberry.type
class Query:
    """Echo query."""

    @strawberry.field(description="Echo query")
    def echo(self, message: str) -> str:
        """
        Sends echo message back to user.

        :param message: incoming message.
        :returns: same message as the incoming.
        """
        return message



from pydantic import BaseModel


class DummyModelDTO(BaseModel):
    """
    DTO for dummy models.

    It returned when accessing dummy models from the API.
    """

    id: int
    name: str

    class Config:
        orm_mode = True


class DummyModelInputDTO(BaseModel):
    """DTO for creating new dummy model."""

    name: str



"""{{cookiecutter.project_name}} models."""
import pkgutil
from pathlib import Path


def load_all_models() -> None:
    """Load all models from this folder."""
    package_dir = Path(__file__).resolve().parent
    modules = pkgutil.walk_packages(
        path=[str(package_dir)],
        prefix="{{cookiecutter.project_name}}.db.models.",
    )
    for module in modules:
        __import__(module.name)  # noqa: WPS421



from typing import Any, Tuple

from sqlalchemy import Table
from sqlalchemy.orm import as_declarative

from {{cookiecutter.project_name}}.db.meta import meta


@as_declarative(metadata=meta)
class Base:
    """
    Base for all models.

    It has some type definitions to
    enhance autocompletion.
    """

    __tablename__: str
    __table__: Table
    __table_args__: Tuple[Any, ...]



import os

from piccolo.conf.apps import AppConfig, table_finder


CURRENT_DIRECTORY = os.path.dirname(os.path.abspath(__file__))


APP_CONFIG = AppConfig(
    app_name='{{cookiecutter.project_name}}_db',
    migrations_folder_path=os.path.join(
        CURRENT_DIRECTORY,
        'migrations'
    ),
    table_classes=table_finder(modules=[
        {%- if cookiecutter.add_dummy == "True" %}
        "{{cookiecutter.project_name}}.db.models.dummy_model"
        {%- endif %}
    ]),
    migration_dependencies=[],
    commands=[]
)
